#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for change dictionary of boundary file
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################
import math,os,glob,sys

from generalSub import *

class changeDictionary:

    def __init__(self,caseDir):
        self.caseDir=caseDir

    def UFile(self,bcDictList,startFrom,startTime,nCore,density,initialDict,notebook,textbuffer):
       
        uini='('+initialDict['uxini']+' '+initialDict['uyini']+' '+initialDict['uzini']+')'

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];Utype=dic['Utype']
            if patchtype=='wall' or patchtype=='patch' or patchtype=='mappedWall':
                if Utype=='no-slip':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
                elif Utype=='fixed-U':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value fixedValue'+opt)
                    valuestr="'("+dic['Ux']+' '+dic['Uy']+' '+dic['Uz']+")'"
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+valuestr+opt)
                elif Utype=='surfaceNormal-U':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value surfaceNormalFixedValue'+opt)
                    if float(dic['Umag']) > 0 :refv='-'+dic['Umag']
                    else:
                        aa=float(dic['Umag'])*-1;refv='-'+str(aa)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry refValue -value '+refv+opt)
                elif Utype=='volume-flowrate':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value flowRateInletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry volumetricFlowRate -value '+dic['volflowrate']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='mass-flowrate':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value flowRateInletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry massFlowRate -value '+dic['massflowrate']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry rho -value rho'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry rhoInlet -value '+density+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='pressureInletOutletVelocity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value pressureInletOutletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry value -value '+"'"+uini+"'"+opt)
                elif Utype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value zeroGradient'+opt)
                elif Utype=='movingWallVelocity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value movingWallVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
                elif Utype=='rotatingWallVelocity':
                    if dic['axis']=='x':axis='(1 0 0)'
                    elif dic['axis']=='y':axis='(0 1 0)'
                    elif dic['axis']=='z':axis='(0 0 1)'
                    elif dic['axis']=='-x':axis='(-1 0 0)'
                    elif dic['axis']=='-y':axis='(0 -1 0)'
                    elif dic['axis']=='-z':axis='(0 0 -1)'
                    origin='('+dic['orix']+' '+dic['oriy']+' '+dic['oriz']+')'
                    omega=float(dic['rpm'])*2*3.141592/60;omega=str(omega)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value rotatingWallVelocity'+opt)                  
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry origin -value '+"'"+origin+"'"+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry axis -value '+"'"+axis+"'"+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry omega -value '+omega+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+" -entry value -value '(0 0 0)'"+opt)
                elif Utype=='cylindricalSwirl':
                    swirlaxis="'("+dic['swirlAxisX']+' '+dic['swirlAxisY']+' '+dic['swirlAxisZ']+")'"
                    swirlcentre="'("+dic['swirlCentreX']+' '+dic['swirlCentreY']+' '+dic['swirlCentreZ']+")'"
                    radialvelocity=dic['swirlRadialVel']
                    swirlangle=dic['swirlAngle']
                    axialvelocity=dic['swirlAxialVel']
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry type -value cylindricalSwirlAngleInletVelocity'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry axis -value '+swirlaxis+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry centre -value '+swirlcentre+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry radialVelocity -value '+radialvelocity+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry swirlAngle -value '+swirlangle+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field U -patch '+name+' -entry axialVelocity -value '+axialvelocity+opt)
                                                      
        if opt==' ':opt='0'

        message='\n    - U Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)          
    #------------------------------------------------------------------------------
    def kFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        kini=initialDict['kini']

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]            
            kkk=dic.keys()
            GEN=generalClass(self.caseDir);ans=GEN.isThereInList(kkk,'ktype')
            if ans=='false':GEN.makeDialog('Error!!! k boundary condition is not set')            
            name=dic['name'];patchtype=dic['type'];ktype=dic['ktype']            
            if patchtype=='wall' or patchtype=='patch':
                if ktype=='wallFunction':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value kqRWallFunction'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
                elif ktype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+dic['kvalue']+opt)
                elif ktype=='intensity':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value turbulentIntensityKineticEnergyInlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry intensity -value '+dic['kvalue']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
                elif ktype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value zeroGradient'+opt)
                elif ktype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry inletValue -value '+dic['kinout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry type -value kqRWallFunction'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field k -patch '+name+' -entry value -value '+kini+opt)
        if opt==' ':opt='0'

        message='\n    - k Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def epsilonFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        eini=initialDict['eini']
        wallfunc='epsilonWallFunction'

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'
                
        for i in range(len(bcDictList)):
            dic=bcDictList[i]            
            kkk=dic.keys()
            GEN=generalClass(self.caseDir);ans=GEN.isThereInList(kkk,'etype')
            if ans=='false':GEN.makeDialog('Error!!! epsilon boundary condition is not set')            
            name=dic['name'];patchtype=dic['type'];etype=dic['etype']
            if patchtype=='wall' or patchtype=='patch':
                if etype=='wallFunction':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value '+wallfunc+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+dic['evalue']+opt)
                elif etype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value zeroGradient'+opt)
                elif etype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry inletValue -value '+dic['einout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)
                elif etype=='mixingLength':
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value turbulentMixingLengthDissipationRateInlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry mixingLength -value '+dic['elength']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry type -value '+wallfunc+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field epsilon -patch '+name+' -entry value -value '+eini+opt)

        if opt==' ':opt='0'
        
        message='\n    - epsilon Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def omegaFile(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        oini=initialDict['oini']
        wallfunc='omegaWallFunction'

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            kkk=dic.keys()
            GEN=generalClass(self.caseDir);ans=GEN.isThereInList(kkk,'otype')
            if ans=='false':GEN.makeDialog('Error!!! omega boundary condition is not set')
            name=dic['name'];patchtype=dic['type'];otype=dic['otype']                       
            if patchtype=='wall' or patchtype=='patch':
                if otype=='wallFunction':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value '+wallfunc+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+oini+opt)
                elif otype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+dic['ovalue']+opt)
                elif otype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value zeroGradient'+opt)
                elif otype=='inletOutlet':
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value inletOutlet'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry inletValue -value '+dic['oinout']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+oini+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry type -value '+wallfunc+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field omega -patch '+name+' -entry value -value '+oini+opt)
        if opt==' ':opt='0'

        message='\n    - omega Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def changeInternalField(self,internalDict,mode,notebook,textbuffer):
    
        f=open(self.caseDir+'/0/U','r');ff=f.readlines();f.close()
        for i in range(len(ff)):
            if ff[i][:13]=='internalField':ff[i]='internalField    uniform ('+internalDict['uxini']+' '+internalDict['uyini']+' '+internalDict['uzini']+'); \n'
        g=open(self.caseDir+'/0/U','w');
        for ii in ff:g.write(ii)
        g.close()
        
        aa=glob.glob(self.caseDir+'/0/k')
        if len(aa)!=0:
            f=open(aa[0],'r');ff=f.readlines();f.close()
            for i in range(len(ff)):
                if ff[i][:13]=='internalField':ff[i]='internalField    uniform '+internalDict['kini']+'; \n'
            g=open(aa[0],'w');
            for ii in ff:g.write(ii)
            g.close()

        aa=glob.glob(self.caseDir+'/0/epsilon')
        if len(aa)!=0:
            f=open(self.caseDir+'/0/epsilon','r');ff=f.readlines();f.close()
            for i in range(len(ff)):
                if ff[i][:13]=='internalField':ff[i]='internalField    uniform '+internalDict['eini']+'; \n'
            g=open(aa[0],'w');
            for ii in ff:g.write(ii)
            g.close()

        aa=glob.glob(self.caseDir+'/0/omega')
        if len(aa)!=0:
            f=open(self.caseDir+'/0/omega','r');ff=f.readlines();f.close()
            for i in range(len(ff)):
                if ff[i][:13]=='internalField':ff[i]='internalField    uniform '+internalDict['oini']+'; \n'
            g=open(aa[0],'w');
            for ii in ff:g.write(ii)
            g.close()

        aa=glob.glob(self.caseDir+'/0/nuTilda')
        if len(aa)!=0:
            f=open(self.caseDir+'/0/nuTilda','r');ff=f.readlines();f.close()
            for i in range(len(ff)):
                if ff[i][:13]=='internalField':ff[i]='internalField    uniform '+internalDict['nuTildaini']+'; \n'
            g=open(aa[0],'w');
            for ii in ff:g.write(ii)
            g.close()

        f=open(self.caseDir+'/0/p_rgh','r');ff=f.readlines();f.close()
        for i in range(len(ff)):
            if ff[i][:13]=='internalField':ff[i]='internalField    uniform '+internalDict['pini']+'; \n'
        g=open(self.caseDir+'/0/p_rgh','w');
        for ii in ff:g.write(ii)
        g.close()
            
        message='\n    - Internal Fields are modified \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0)                                            
    #------------------------------------------------------------------------------
    def p_rghFileVOF(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        pini=initialDict['pini']

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];ptype=dic['ptype']
            if patchtype=='wall' or patchtype=='patch':
                if ptype=='totalPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value totalPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry gamma -value 1'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry p0 -value '+dic['ptotal']+opt)
                elif ptype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value '+dic['pstatic']+opt)
                elif ptype=='alphaWeighted':pass
                else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedFluxPressure'+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value fixedFluxPressure'+opt)
                os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value '+pini+opt)
            elif patchtype=='cyclic':
                if ptype=='porousJump':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value porousJumpPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry patchType -value cyclic'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry value -value 0'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry I -value '+dic['jumpi']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry length -value '+dic['jumpl']+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry D -value '+dic['jumpd']+opt)
            	else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p_rgh -patch '+name+' -entry type -value cyclic'+opt)

        if opt==' ':opt='0'

        message='\n    - p_rgh Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
        notebook.set_current_page(0) 
    #------------------------------------------------------------------------------
    def pFilePotential(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        pini=initialDict['pini']

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];ptype=dic['ptype']                       
            if patchtype=='wall' or patchtype=='patch':
                if ptype=='totalPressure':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name[i]+' -entry type -value totalPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name[i]+' -entry gamma -value 1'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry p0 -value '+BCDict[name][5][0]+opt)
                elif ptype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value '+dic['pstatic']+opt)
                else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value zeroGradient'+opt)
            elif patchtype=='mappedWall':
                os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value zeroGradient'+opt)
            elif patchtype=='cyclic':
                if ptype=='porousJump':
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value porousJumpPressure'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry patchType -value cyclic'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry value -value 0'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry I -value '+jumpi+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry length -value '+jumpl+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry D -value '+jumpd+opt)
            	else:
                    os.system('changeBCs -case '+self.caseDir+ ' -field p -patch '+name+' -entry type -value cyclic'+opt)
        if opt==' ':opt='0'

        textbuffer.insert_at_cursor('\n -- p Boundary Conditions are changed for potentialFoam.\n')
        notebook.set_current_page(0)         
    #------------------------------------------------------------------------------
    def alphaFileVOF(self,bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer):

        alphaini=initialDict['alphaini']

        GEN=generalClass(self.caseDir);results=GEN.getResult()
        aa=glob.glob(self.caseDir+'/processor*')
        if len(aa)==0:
            if results[0]==[]:opt=''
            else:opt=' -latestTime'
        else:
            if nCore!='1':
                if results[1]==[]:opt=' -parallel'
                else:opt=' -latestTime -parallel'
            else:
                if results[0]==[]:opt=''
                else:opt=' -latestTime'

        for i in range(len(bcDictList)):
            dic=bcDictList[i]
            name=dic['name'];patchtype=dic['type'];Ttype=dic['alphatype']
            if patchtype=='wall' or patchtype=='patch' or patchtype=='mappedWall':
                if Ttype=='zeroGradient':
                    os.system('changeBCs -case '+self.caseDir+ ' -field alpha.water -patch '+name+' -entry type -value zeroGradient'+opt)
                elif Ttype=='fixedValue':
                    os.system('changeBCs -case '+self.caseDir+ ' -field alpha.water -patch '+name+' -entry type -value fixedValue'+opt)
                    os.system('changeBCs -case '+self.caseDir+ ' -field alpha.water -patch '+name+' -entry value -value '+dic['alphavalue']+opt)

        if opt==' ':opt='0'

        message='\n    - alpha.water Boundary Conditions are set at time '+opt+'  \n'
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)        
        notebook.set_current_page(0) 


