#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for writing various files
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import math,os,glob,sys

from generalSub import *

class writeFileClass:

    def __init__(self,caseDir):
        self.caseDir=caseDir
	
    def mrfFile(self,dic):
        cellZone=dic['cellZoneName']
        aa=dic['nonRotPatchList'];nonRotPatch=aa.split(' ')
        orix=dic['orix'];oriy=dic['oriy'];oriz=dic['oriz']
        rpm=dic['rpm'];rotAxis=dic['axis']
	    
        if rotAxis=='x':axis='(1 0 0)'
        elif rotAxis=='y':axis='(0 1 0)'
        elif rotAxis=='z':axis='(0 0 1)'
        elif rotAxis=='-x':axis='(-1 0 0)'
        elif rotAxis=='-y':axis='(0 -1 0)'
        elif rotAxis=='-z':axis='(0 0 -1)'
        
        omega=float(rpm)*2*3.141592/60
	    
        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':MRF','w')
        f.write('MRFCellZone_'+cellZone+'\n')
        f.write('{\n')
        f.write('    type            MRFSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    MRFSourceCoeffs\n')
        f.write('    {\n')
        f.write('        nonRotatingPatches    (')
        for jj in nonRotPatch:f.write(' '+jj+' ')
        f.write(' );\n')
        f.write('        origin     ('+orix+' '+oriy+' '+oriz+');\n')
        f.write('        axis       '+axis+';\n')
        f.write('        omega      '+str(omega)+';\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()
    #-------------------------------------------------------------------------------------        
    def porousFile(self,dic):
    
        cellZone=dic['cellZoneName']
        potype=dic['porousType']    
        ax=dic['xDarcy'];ay=dic['yDarcy'];az=dic['zDarcy']
        bx=dic['xForch'];by=dic['yForch'];bz=dic['zForch']
        e1x=dic['e1x'];e1y=dic['e1y'];e1z=dic['e1z']
        e2x=dic['e2x'];e2y=dic['e2y'];e2z=dic['e2z']	
        c0=dic['powerC0'];c1=dic['powerC1']

        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Porous','w')
        f.write('porous_'+cellZone+'\n')
        f.write('{\n')
        f.write('    type            explicitPorositySource;\n')
        f.write('    active          true;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    explicitPorositySourceCoeffs\n')
        f.write('    {\n')
        f.write('        type            '+potype+';\n')
        f.write('        active          yes;\n')
        f.write('\n')
        if potype=='DarcyForchheimer':
            f.write('        DarcyForchheimerCoeffs\n')
            f.write('        {\n')
            f.write('            d       d [0 -2 0 0 0 0 0] ('+ax+' '+ay+' '+az+');\n')
            f.write('            f       f [0 -1 0 0 0 0 0] ('+bx+' '+by+' '+bz+');\n')
            f.write('\n')
            f.write('            coordinateSystem\n')
            f.write('            {\n')
            f.write('                type    cartesian;\n')
            f.write('                origin  (0 0 0);\n')
            f.write('                coordinateRotation\n')
            f.write('                {\n')
            f.write('                    type    axesRotation;\n')
            f.write('                    e1  ('+e1x+' '+e1y+' '+e1z+');\n')
            f.write('                    e2  ('+e2x+' '+e2y+' '+e2z+');\n')
            f.write('                }\n')
            f.write('            }\n')
            f.write('        }\n')
        else:
            f.write('        powerLawCoeffs\n')
            f.write('        {\n')
            f.write('            C0      '+c0+';\n')
            f.write('            C1      '+c1+';\n')
            f.write('\n')
            f.write('            coordinateSystem\n')
            f.write('            {\n')
            f.write('                type    cartesian;\n')
            f.write('                origin  (0 0 0);\n')
            f.write('                coordinateRotation\n')
            f.write('                {\n')
            f.write('                    type    axesRotation;\n')
            f.write('                    e1  ('+e1x+' '+e1y+' '+e1z+');\n')
            f.write('                    e2  ('+e2x+' '+e2y+' '+e2z+');\n')
            f.write('                }\n')
            f.write('            }\n')
            f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')               
        f.close()       
    #-------------------------------------------------------------------------------------
    def momSourceFile(self,dic):
    
        cellZone=dic['cellZoneName']
        ux=dic['momSourceX'];uy=dic['momSourceY'];uz=dic['momSourceZ']

        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Mom.Source','w')
        f.write('fixedU_'+cellZone+'\n')
        f.write('{\n')	
        f.write('    type            pressureGradientExplicitSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    pressureGradientExplicitSourceCoeffs\n')
        f.write('    {\n')
        f.write('        fieldNames ( U );\n')
        f.write('        Ubar    ('+ux+' '+uy+' '+uz+');\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()
    #-------------------------------------------------------------------------------------
    def fixedUFile(self,dic):
    
        cellZone=dic['cellZoneName']
        ux=dic['fixedUx'];uy=dic['fixedUy'];uz=dic['fixedUz']        
        
        f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':Fixed.U','w')
        f.write('fixedU_'+cellZone+'\n')
        f.write('{\n')	
        f.write('    type            pressureGradientExplicitSource;\n')
        f.write('    active          yes;\n')
        f.write('    selectionMode   cellZone;\n')
        f.write('    cellZone        '+cellZone+';\n')
        f.write('\n')
        f.write('    pressureGradientExplicitSourceCoeffs\n')
        f.write('    {\n')
        f.write('        fieldNames    (U);\n')
        f.write('        Ubar          ('+ux+' '+uy+' '+uz+');\n')
        f.write('        gradPini      gradPini [0 2 -2 0 0] 0;\n')
        f.write('    }\n')
        f.write('}\n')
        
        if dic['energySourceOnOff']=='true':
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
        f.close()  
    #-------------------------------------------------------------------------------------
    def noneFile(self,dic):
    
        cellZone=dic['cellZoneName']        
       
        if dic['energySourceOnOff']=='true':
            f=open(self.caseDir+'/system/cellZoneConditions:'+cellZone+':None','w')
            if dic['eSourceMethod']=='Fixed.T':
                f.write('fixedT_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            fixedTemperatureConstraint;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    fixedTemperatureConstraintCoeffs\n')
                f.write('    {\n')
                f.write('        mode           uniform;\n')
                f.write('        temperature    '+dic['eSource']+';\n')
                f.write('    }\n')
                f.write('}\n')
            else:
                f.write('energySource_'+cellZone+'\n')
                f.write('{\n')
                f.write('    type            scalarSemiImplicitSource;\n')
                f.write('    active          yes;\n')
                f.write('    selectionMode   cellZone;\n')
                f.write('    cellZone        '+cellZone+';\n')
                f.write('\n')
                f.write('    scalarSemiImplicitSourceCoeffs\n')
                f.write('    {\n')
                f.write('        volumeMode     '+dic['eSourceMethod']+';\n')
                f.write('        injectionRateSuSp\n')
                f.write('        {\n')
                f.write('            h    ('+dic['eSource']+' 0);\n')
                f.write('        }\n')
                f.write('    }\n')
                f.write('}\n')        
            f.close()                                
    #-------------------------------------------------------------------------------------
    def foamComment(self):
        fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
        return fc
    #------------------------------------------------------------------------------
    def header(self,location,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "'+location+'";\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def BCheader(self,classtype,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       '+classtype+';\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def topoSetDict(self,sets):
        fcom=self.foamComment();head=self.header('system','topoSetDict');hhh=fcom+head
        f=open(self.caseDir+'/system/topoSetDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('actions\n')
        f.write('(\n')
        for ss in sets:
            f.write('    {\n')
            f.write('        name    '+ss+';\n')
            f.write('        type    faceZoneSet;\n')
            f.write('        action  new;\n')
            f.write('        source  setToFaceZone;\n')
            f.write('        sourceInfo\n')
            f.write('        {\n')
            f.write('            faceSet '+ss+';\n')
            f.write('        }\n')
            f.write('    }\n')
        f.write(');\n')
        f.close()
    #------------------------------------------------------------------------------
    def createBafflesDict(self,sets):
        fcom=self.foamComment()
        head=self.header('system','createBafflesDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('internalFacesOnly true;\n')
        f.write('baffles\n')
        f.write('{\n')
        for ss in sets:                  
            f.write('    '+ss+'\n')
            f.write('    {\n')
            f.write('        type        faceZone;\n')
            f.write('        zoneName    '+ss+';\n')
            f.write('        patchPairs\n')
            f.write('        {\n')
            f.write('            type	mappedWall;\n')
            f.write('            sampleMode	nearestPatchFace;\n')
            f.write('	     }\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def createInteriorDict(self,sets):
        fcom=self.foamComment()
        head=self.header('system','createBafflesDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('internalFacesOnly true;\n')
        f.write('baffles\n')
        f.write('{\n')
        for ss in sets:                  
            f.write('    '+ss+'\n')
            f.write('    {\n')
            f.write('        type        faceZone;\n')
            f.write('        zoneName    '+ss+';\n')
            f.write('        patchPairs\n')
            f.write('        {\n')
            f.write('            type        cyclic;\n')
            f.write('	     }\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()                   
    #------------------------------------------------------------------------------
    def changeDictionaryDictNoCyclicAMI(self,patch,newtype):
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDict')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        if newtype!='cyclicAMI':
            f.write('        '+patch+'\n')
            f.write('        {\n')
            f.write('            type    '+newtype+';\n')
            f.write('        }\n')			
        f.write('    }\n')
        f.write('}\n')
        f.close() 
    #------------------------------------------------------------------------------
    def changeDictionaryDictInterface(self,selectedFaces,weight,weightValue):        
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictInterface')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictInterface','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')

        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+selectedFaces[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            inGroups          1(cyclicAMI);\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         noOrdering;\n')
        f.write('            neighbourPatch    '+selectedFaces[1]+';\n')
        if weight==True:f.write('            lowWeightCorrection    '+weightValue+';\n')
        f.write('        }\n')
        f.write('        '+selectedFaces[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            inGroups          1(cyclicAMI);\n')        
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         noOrdering;\n')
        f.write('            neighbourPatch    '+selectedFaces[0]+';\n')
        if weight==True:f.write('            lowWeightCorrection    '+weightValue+';\n')
        f.write('        }\n')			
        f.write('    }\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def changeDictionaryDictRotCyclic(self,sets,cc1,cc2,cc3,axis):          
        
        if axis=='x':raxis='(1 0 0)'
        elif axis=='y':raxis='(0 1 0)'
        elif axis=='z':raxis='(0 0 1)'

        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictRotCyclic')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictRotCyclic','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')     
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()                      
    #------------------------------------------------------------------------------
    def changeDictionaryDictTranslateCyclic(self,sets,cc1,cc2,cc3):         

        vector1='('+cc1+' '+cc2+' '+cc3+')'
        xx=-1*float(cc1);yy=-1*float(cc2);zz=-1*float(cc3)
        vector2='('+str(xx)+' '+str(yy)+' '+str(zz)+')'
	
        fcom=self.foamComment()
        head=self.header('system','changeDictionaryDictTranslateCyclic')
        hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictTranslateCyclic','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')          
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            separationVector  '+vector1+';\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            separationVector  '+vector2+';\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()          
