#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for boundary condition free-panel 
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import os,math,sys;import pygtk;pygtk.require('2.0');import gtk;import glob

from generalSub import *
from boundaryConditionSub import *
from writeFileSub import *

class freepanelBoundaryConditionClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,mainwindow,installpath):
        self.caseDir=caseDir
        self.mainwindow=mainwindow
        self.installpath=installpath

    def boundaryCondition(self,gvtk,notebook,textbuffer):
    
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        self.boundarybox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  Boundary Conditions  </b>')
        ebox=gtk.EventBox();self.boundarybox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label) 

        hbox=gtk.HBox(False,5);self.boundarybox.pack_start(hbox,False,False,5)
        bcswin=gtk.ScrolledWindow();bcswin.set_border_width(0);bcswin.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_ALWAYS)
        hbox.pack_start(bcswin,True,True,0);bcitembox=gtk.VBox(False,0);bcswin.add_with_viewport(bcitembox)
        bcswin.set_size_request(200,640)

        self.makeBCBox(bcitembox,self.boundarybox,gvtk,notebook,textbuffer)

        return self.boundarybox
    #---------------------------------------------------------------------------------------------------------        
    def makeBCBox(self,bcitembox,boundarybox,gvtk,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        if glob.glob(self.caseDir+'/constant/polyMesh/boundary')==[]:
            GEN.makeDialog('Error!!! There might be some problem with mesh')
            return
            
        bcnameList=GEN.getBCName();bctypeList=GEN.getBCType()

        def detail(widget,path,column):
            model=self.treeview.get_model()
            iter=model.get_iter(path[0])
            namepatch=model.get_value(iter,0)
            typepatch=model.get_value(iter,1) 
            
            if typepatch=='wall' or typepatch=='patch':
                BCS=boundaryConditionClass(self.caseDir,self.installpath)
                BCS.openBCWindow(namepatch,typepatch,gvtk,self.mainwindow,notebook,textbuffer,self.liststore_patchname)            
                f=open(self.caseDir+'/system/settings/openSubWindow','w');f.close()
            else:GEN.makeDialog('There is nothing to set with '+typepatch+' patch')

        def changebctype(widget, path, text):        
            bcnameList=GEN.getBCName();bctypeList=GEN.getBCType()
            
            if bctypeList[int(path)]!=text:
                self.liststore_patchname[path][1] = text
                patch=bcnameList[int(path)]
                newtype=text
                curtype=bctypeList[int(path)]
                if newtype=='cyclic' or newtype=='mappedWall' or newtype=='empty':
                    GEN.makeDialog('B.C. type cannot be changed to '+newtype)
                    self.liststore_patchname[path][1] = curtype
                else:            
                    self.changepatchtype(bcnameList,bctypeList,patch,newtype,curtype,bcitembox,boundarybox,gvtk,notebook,textbuffer)

        self.liststore_types = gtk.ListStore(str)
        self.btypeList=['wall','patch','symmetry','symmetryPlane','empty']
        for ii in self.btypeList:self.liststore_types.append([ii])

        self.liststore_patchname = gtk.ListStore(str, str)
        for i in range(len(bcnameList)): 
            self.liststore_patchname.append([bcnameList[i],bctypeList[i]])

        self.treeview = gtk.TreeView(model=self.liststore_patchname)       
        self.treeview.connect('row-activated',detail)

        renderer_text = gtk.CellRendererText()
        
        column_text = gtk.TreeViewColumn("patch name                ", renderer_text, text=0)
        self.treeview.append_column(column_text)

        renderer_combo = gtk.CellRendererCombo()
        renderer_combo.set_property("editable", True)
        renderer_combo.set_property("model", self.liststore_types)
        renderer_combo.set_property("text-column", 0)
        renderer_combo.set_property("has-entry", False)
        renderer_combo.connect("edited", changebctype)

        column_combo = gtk.TreeViewColumn("type", renderer_combo, text=1)

        column_text.set_sort_column_id(0)
        column_combo.set_sort_column_id(1)

        self.treeview.append_column(column_combo)
        
        bcitembox.pack_start(self.treeview,True,True,2) 
    #--------------------------------------------------------------------------------------------------------- 
    def changepatchtype(self,bcnameList,bctypeList,patch,newtype,curtype,bcitembox,boundarybox,gvtk,notebook,textbuffer):
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)
        if newtype=='cyclicAMI':                            
            if curtype!='cyclicAMI':self.selectAMIType(bcnameList,bctypeList,patch,bcitembox,boundarybox,gvtk,notebook,textbuffer)
        else:
            if curtype!='cyclicAMI':
                WFILE.changeDictionaryDictNoCyclicAMI(patch,newtype)
                os.system('changeDictionary -disablePatchGroups -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.changeDictionary')
                message='\n [  Patch type of "'+patch+'" is changed to '+newtype+'  ]\n\n'                     
                enditer=textbuffer.get_end_iter()
                textbuffer.insert(enditer,message)
                
            else:self.changeAMIToOthers(patch,newtype)

        aa=glob.glob(self.caseDir+'/system/settings/BCDict:'+patch)
        if len(aa)!=0:
            for ii in aa:os.system('rm '+ii)
                            
        self.refresh(bcitembox,boundarybox,gvtk,notebook,textbuffer)
    #--------------------------------------------------------------------------------------------------------- 
    def changepatchname(self,oldname,newname,bcitembox,boundarybox,gvtk,notebook):
        GEN=generalClass(self.caseDir);WFILE=writeFileClass(self.caseDir)

        f=open(self.caseDir+'/constant/polyMesh/boundary','r');ff=f.readlines();f.close()
        for i in range(len(ff)):
            if ff[i][4:-1]==oldname:ff[i]='    '+newname+'\n'
        g=open(self.caseDir+'/constant/polyMesh/boundary','w')
        for ii in ff:g.write(ii)
        g.close()
        
        os.system('mv '+self.caseDir+'/VTK/'+oldname+' '+self.caseDir+'/VTK/'+newname)
        os.system('mv '+self.caseDir+'/VTK/'+newname+'/'+oldname+'_0.vtk '+self.caseDir+'/VTK/'+newname+'/'+newname+'_0.vtk')

        aa=glob.glob(self.caseDir+'/system/settings/BCDict:'+oldname)
        if len(aa)!=0:
            for ii in aa:os.system('rm '+ii)        
    #---------------------------------------------------------------------------------------------------------                        
    def refresh(self,bcitembox,boundarybox,gvtk,notebook,textbuffer):
        aa=bcitembox.get_children()
        for ii in aa:bcitembox.remove(ii)
        self.makeBCBox(bcitembox,boundarybox,gvtk,notebook,textbuffer)
        boundarybox.show_all()                         
    #---------------------------------------------------------------------------------------------------------
    def boundarySort(self):
        GEN=generalClass(self.caseDir)
        bn=GEN.getBCName();bt=GEN.getBCType()
        bdict={}
        for i in range(len(bn)):bdict[bn[i]]=bt[i]
        
        patches=[];walls=[];amis=[];etcs=[]
        for ii in bn:
            if bdict[ii]=='patch':patches.append(ii)
            elif bdict[ii]=='wall':walls.append(ii)
            else:etcs.append(ii)
        
        patches.sort();walls.sort();amis.sort();etcs.sort()    
        names=[];types=[]
        for ii in patches:names.append(ii);types.append(bdict[ii])
        for ii in walls:names.append(ii);types.append(bdict[ii])
        for ii in amis:names.append(ii);types.append(bdict[ii])
        for ii in etcs:names.append(ii);types.append(bdict[ii])

        nametypelist=[names,types]       
        return nametypelist                
    #---------------------------------------------------------------------------------------------------------


    
    

