#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for gravity, operating condition, initial condition free-panel
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import os,math,sys;import pygtk;pygtk.require('2.0');import gtk;import glob

from generalSub import *
from boundaryConditionSub import *
from writeFileSub import *

class freepanelInternalFieldClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,mainwindow,installpath):
        self.caseDir=caseDir
        self.mainwindow=mainwindow
        self.installpath=installpath     
    #--------------------------------------------------------------------------------------  
    def initialCondition(self,notebook,textbuffer):
    
        GEN=generalClass(self.caseDir)        

        aa=glob.glob(self.caseDir+'/system/settings/generalConditions')
        if len(aa)==0:
            GEN.makeDialog('Error!!! Sim. conditions is not set.')
            return
        else: 
            genDict=GEN.makeDictFromFile(aa[0])
            self.speed=genDict['speed'] 
            self.inletk=genDict['inletk']
            self.inlete=genDict['inlete']            
            gendict=GEN.makeDictFromFile(aa[0])
            turmodel=gendict['RANSModel']

        def applyini(widget):
            GEN=generalClass(self.caseDir)
            initialDict={}
            initialDict['uxini']=self.uxiniEntry.get_text();initialDict['uyini']=self.uyiniEntry.get_text();initialDict['uzini']=self.uziniEntry.get_text()
            initialDict['pini']=self.piniEntry.get_text();initialDict['alphaini']=self.alphainiEntry.get_text()
            initialDict['kini']=self.kiniEntry.get_text();initialDict['eini']=self.einiEntry.get_text()
            initialDict['oini']=self.oiniEntry.get_text()
            
            if GEN.isNumber(initialDict['uxini'])==False:GEN.makeDialog('Error!!! ux is not number')
            if GEN.isNumber(initialDict['uyini'])==False:GEN.makeDialog('Error!!! uy is not number')
            if GEN.isNumber(initialDict['uzini'])==False:GEN.makeDialog('Error!!! uz is not number')
            if GEN.isNumber(initialDict['pini'])==False:GEN.makeDialog('Error!!! p is not number')
            if GEN.isNumber(initialDict['alphaini'])==False:GEN.makeDialog('Error!!! alpha is not number')
            if GEN.isNumber(initialDict['kini'])==False:GEN.makeDialog('Error!!! k is not number')
            if GEN.isNumber(initialDict['eini'])==False:GEN.makeDialog('Error!!! epsilon is not number')
            if GEN.isNumber(initialDict['oini'])==False:GEN.makeDialog('Error!!! omega is not number')
            
            keys=initialDict.keys()
            f=open(self.caseDir+'/system/settings/initialConditions','w')
            for ii in keys:f.write(ii+';'+initialDict[ii]+'\n')
            f.close()
            
            message='\n [  Internal field is saved  ]\n'
            message=message+'\n    - internalField of U is ( '+initialDict['uxini']+' '+initialDict['uyini']+' '+initialDict['uzini']+' ) \n'  
            message=message+'\n    - internalField of p is '+initialDict['pini']+'\n'
            message=message+'\n    - internalField of k is '+initialDict['kini']+'\n'
            message=message+'\n    - internalField of epsilon/omega is '+initialDict['eini']+'\n'
            message=message+'\n    - internalField of alpha is '+initialDict['alphaini']+'\n'                
            enditer=textbuffer.get_end_iter()
            textbuffer.insert(enditer,message)
            
            notebook.set_current_page(0)   
                        
        def setSavedValue():
            aa=glob.glob(self.caseDir+'/system/settings/initialConditions')
            if len(aa)!=0:
                dic=GEN.makeDictFromFile(aa[0])
                self.uxiniEntry.set_text(dic['uxini']);self.uyiniEntry.set_text(dic['uyini']);self.uziniEntry.set_text(dic['uzini'])
                self.piniEntry.set_text(dic['pini']);self.alphainiEntry.set_text(dic['alphaini'])          
                self.kiniEntry.set_text(dic['kini'])
                self.einiEntry.set_text(dic['eini']);self.oiniEntry.set_text(dic['oini'])
        
        self.initialbox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  Initial Conditions  </b>')
        ebox=gtk.EventBox();self.initialbox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label) 

        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        label=gtk.Label('U [m/s]');hbox.pack_start(label,False,False,5)
        self.uxiniEntry=gtk.Entry();self.uxiniEntry.set_text(self.speed);self.uxiniEntry.set_size_request(50,28)
        self.uyiniEntry=gtk.Entry();self.uyiniEntry.set_text('0');self.uyiniEntry.set_size_request(50,28)
        self.uziniEntry=gtk.Entry();self.uziniEntry.set_text('0');self.uziniEntry.set_size_request(50,28)
        hbox.pack_end(self.uziniEntry,False,False,0);hbox.pack_end(self.uyiniEntry,False,False,0);hbox.pack_end(self.uxiniEntry,False,False,0)
        
        self.piniEntry=gtk.Entry();self.piniEntry.set_text('0');self.piniEntry.set_size_request(80,28)
        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        label=gtk.Label('p [m2/s2]');hbox.pack_start(label,False,False,5);label.set_size_request(120,28);label.set_alignment(0,0.5)
        hbox.pack_end(self.piniEntry,False,False,0)
        
        self.alphainiEntry=gtk.Entry();self.alphainiEntry.set_text('0');self.alphainiEntry.set_size_request(80,28)
        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)        
        label=gtk.Label('alpha');hbox.pack_start(label,False,False,5);label.set_size_request(120,28);label.set_alignment(0,0.5)
        hbox.pack_end(self.alphainiEntry,False,False,0)
        
        self.kiniEntry=gtk.Entry();self.kiniEntry.set_text(self.inletk);self.kiniEntry.set_size_request(80,28)
        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        label=gtk.Label('k [m2/s2]');hbox.pack_start(label,False,False,5);label.set_size_request(120,28);label.set_alignment(0,0.5)
        hbox.pack_end(self.kiniEntry,False,False,0)
        
        self.einiEntry=gtk.Entry();self.einiEntry.set_text(self.inlete);self.einiEntry.set_size_request(80,28)
        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        label=gtk.Label('epsilon [m2/s3]');hbox.pack_start(label,False,False,5);label.set_size_request(140,28);label.set_alignment(0,0.5)
        hbox.pack_end(self.einiEntry,False,False,0)
        
        self.oiniEntry=gtk.Entry();self.oiniEntry.set_text(self.inlete);self.oiniEntry.set_size_request(80,28)
        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        label=gtk.Label('omega [1/s]');hbox.pack_start(label,False,False,5);label.set_size_request(140,28);label.set_alignment(0,0.5)
        hbox.pack_end(self.oiniEntry,False,False,0)

        hbox=gtk.HBox(False,5);self.initialbox.pack_start(hbox,False,False,5)
        self.initialAppB=gtk.Button('Apply');self.initialAppB.connect('clicked',applyini);hbox.pack_end(self.initialAppB,True,True,5)
              
        # for turbulence model
        if turmodel=='kEpsilon' or turmodel=='RNGKE' or turmodel=='realizableKE':
            self.oiniEntry.set_sensitive(0);self.kiniEntry.set_sensitive(1);self.einiEntry.set_sensitive(1)
        elif turmodel=='kOmega' or turmodel=='kOmegaSST':
            self.oiniEntry.set_sensitive(1);self.kiniEntry.set_sensitive(1);self.einiEntry.set_sensitive(0)
        
        setSavedValue()

        return self.initialbox
    #--------------------------------------------------------------------------------------  




