#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for snappyHexMesh free-panel
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import os,math,sys;import pygtk;pygtk.require('2.0');import gtk;import glob
import time

from generalSub import *
from boundaryConditionSub import *
from snappySub import *
from VTKStuffClass import *
from writeFileResistanceSub import *

class freepanelSnappyClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,mainwindow,installpath):
        self.caseDir=caseDir
        self.mainwindow=mainwindow
        self.installpath=installpath
        self.openSubWindow=False
        self.rr=1;self.gg=1;self.bb=1
        
    def snappyHexMesh(self,gvtk,notebook,textbuffer,terminal,displayOptCombo):
    
        GEN=generalClass(self.caseDir);SNAPPY=snappySubClass(self.caseDir);VTK=VTKStuff(self.caseDir,self.rr,self.gg,self.bb)

        aa=glob.glob(self.caseDir+'/system/settings/generalConditions')
        if len(aa)!=0:
            genDict=GEN.makeDictFromFile(aa[0])
            self.speed=genDict['speed']
            self.lbp=genDict['lbp']
                    
        aa=glob.glob(self.caseDir+'/system/settings/generalConditions')
        if len(aa)==0:
            GEN.makeDialog('Error!!! Sim. conditions is not set.')
            return
        
        self.advancedValue=[['20000000','20000000','30','true'],['false','true','true'],['0','5','30','10','10','0.5','0.3','90','0','30','40'],
                            ['75','20','4','80','1e-13','1e-15','-1','0.02','0.001','0.05','0.01','-1','4','0.75','75','0','1e-6']]
        self.hostfilename='None'

        def saveSnappy():            
            snappyDict={}
            
            snappyDict['waterline']=self.waterlineEntry.get_text()
            snappyDict['minx']=self.minxEntry.get_text();snappyDict['miny']=self.minyEntry.get_text();snappyDict['minz']=self.minzEntry.get_text()
            snappyDict['maxx']=self.maxxEntry.get_text();snappyDict['maxy']=self.maxyEntry.get_text();snappyDict['maxz']=self.maxzEntry.get_text()
            snappyDict['nodex']=self.nodexEntry.get_text();snappyDict['nodey']=self.nodeyEntry.get_text();snappyDict['nodez']=self.nodezEntry.get_text()            
            snappyDict['nCellsBetweenLevels']=self.betweenEntry.get_text()
            snappyDict['tolerance']=self.tolEntry.get_text()
            snappyDict['nSolverIter']=self.nsolveEntry.get_text()
            snappyDict['nRelaxIter']=self.nrelaxEntry.get_text()            
            snappyDict['nSurfaceLayers']=self.nlayerEntry.get_text()
            snappyDict['expansionRatio']=self.expandlayerEntry.get_text()
            snappyDict['firstLayerThickness']=self.firstlayerEntry.get_text()
            snappyDict['nCores']=self.coreEntry.get_text()

            if GEN.isNumber(snappyDict['waterline'])==False:GEN.makeDialog('Error!!! waterline is not number')   
            if GEN.isNumber(snappyDict['minx'])==False:GEN.makeDialog('Error!!! minx is not number')            
            if GEN.isNumber(snappyDict['miny'])==False:GEN.makeDialog('Error!!! miny is not number')            
            if GEN.isNumber(snappyDict['minz'])==False:GEN.makeDialog('Error!!! minz is not number') 
            if GEN.isNumber(snappyDict['maxx'])==False:GEN.makeDialog('Error!!! maxx is not number') 
            if GEN.isNumber(snappyDict['maxy'])==False:GEN.makeDialog('Error!!! maxy is not number') 
            if GEN.isNumber(snappyDict['maxz'])==False:GEN.makeDialog('Error!!! maxz is not number') 
            if GEN.isNumber(snappyDict['nodex'])==False:GEN.makeDialog('Error!!! nodex is not number') 
            if GEN.isNumber(snappyDict['nodey'])==False:GEN.makeDialog('Error!!! nodey is not number') 
            if GEN.isNumber(snappyDict['nodez'])==False:GEN.makeDialog('Error!!! nodez is not number')            
            if GEN.isNumber(snappyDict['nCellsBetweenLevels'])==False:GEN.makeDialog('Error!!! nCellsBetweenLevels is not number') 
            if GEN.isNumber(snappyDict['tolerance'])==False:GEN.makeDialog('Error!!! tolerance is not number') 
            if GEN.isNumber(snappyDict['nSolverIter'])==False:GEN.makeDialog('Error!!! nSolverIter is not number') 
            if GEN.isNumber(snappyDict['nRelaxIter'])==False:GEN.makeDialog('Error!!! nRelaxIter is not number')             
            if GEN.isNumber(snappyDict['nSurfaceLayers'])==False:GEN.makeDialog('Error!!! nSurfaceLayers is not number') 
            if GEN.isNumber(snappyDict['expansionRatio'])==False:GEN.makeDialog('Error!!! expansionRatio is not number') 
            if GEN.isNumber(snappyDict['firstLayerThickness'])==False:GEN.makeDialog('Error!!! firstLayerThickness is not number') 
            if GEN.isNumber(snappyDict['nCores'])==False:GEN.makeDialog('Error!!! No. of cores is not number')           
                        
            keys=snappyDict.keys()
            f=open(self.caseDir+'/system/settings/snappyConditions','w')
            for ii in keys:f.write(ii+';'+snappyDict[ii]+'\n')         
            f.close()
            
            return snappyDict
            
        def createMesh(widget): 
            
            GEN=generalClass(self.caseDir)           
            
            snappyDict=saveSnappy()

            generalMesh=[['true','true','true','true'],'0',self.coreEntry.get_text(),'None']
            xx=float(self.minxEntry.get_text())+0.1;yy=float(self.minyEntry.get_text())+0.1;zz=float(self.minzEntry.get_text())+0.1
            casMesh=[[str(xx),str(yy),str(zz)],'2',self.betweenEntry.get_text()]
            snapMesh=['3',self.tolEntry.get_text(),self.nsolveEntry.get_text(),self.nrelaxEntry.get_text(),'10']
            layerMesh=['false',self.nlayerEntry.get_text(),self.expandlayerEntry.get_text(),'1e-7',self.firstlayerEntry.get_text(),'120']
            #generalMesh=[casSnapLayer,startFrom,nCore,self.hostfilename]
	        #casMesh=[locationInMesh,minRefinementCells,nCellsBetweenLevels]
	        #snapMesh=[nSmoothPatch,tolerance,nSolveIter,nRelaxIter,nFeatureSnapIter]
	        #layerMesh=[relativeSizes,layers,expansionRatio,minThickness,finalLayerThickness,featureAngle]

            stlfiles=[];featuresizelevel=[]
            aa=glob.glob(self.caseDir+'/system/settings/featureLevel.txt')
            if len(aa)!=0:
                f=open(aa[0],'r');ff=f.readlines();f.close()
                for i in range(len(ff)):
                    aa=ff[i][:-1].split(' ');stlfiles.append(aa[0]);featuresizelevel.append(aa[1])
            else:
                f=open(self.caseDir+'/system/settings/stlfiles','r');ff=f.readlines();f.close()
                for i in range(len(ff)):
                    aa=ff[i].split('/');stlfiles.append(aa[-1]);featuresizelevel.append('3')

            notebook.set_current_page(2)

            sf=open(self.caseDir+'/runSnappy','w')
            
            featureAngle='160'
            SNAPPY.makeFeatureFile(stlfiles,featureAngle)
            sf.write('surfaceFeatureExtract -case '+self.caseDir+'\n')
            
            surfacefilename=[];surfaceregion=[];surfacesizelevel=[]
            aa=glob.glob(self.caseDir+'/system/settings/surfaceLevel.txt')
            if len(aa)!=0:
                f=open(aa[0],'r');ff=f.readlines();f.close()
                for i in range(len(ff)):
                    aa=ff[i][:-1].split(' ');surfacesizelevel.append(aa[1])
            else:
                f=open(self.caseDir+'/system/settings/stlfiles','r');ff=f.readlines();f.close()
                for i in range(len(ff)):
                    aa=ff[i][:-1].split('/');surfacesizelevel.append('3')

            surfNfeat=[stlfiles,featuresizelevel,surfacesizelevel]

            refinebox={}
            aa=glob.glob(self.caseDir+'/system/settings/box-*')
            if len(aa)!=0:
                for i in range(len(aa)):
                    b1=aa[i].split('/')
                    f=open(aa[i],'r');ff=f.readlines();f.close()
                    refinebox[b1[-1]]=ff

            aa=glob.glob(self.caseDir+'/system/settings/snappyAdvancedSetup')
            if len(aa)!=0:
                snappyAdvDict=GEN.makeDictFromFile(aa[0])
            else:
                snappyAdvDict=SNAPPY.defaultSnappyAdvDict()
                
            SNAPPY.makeSnappyFileVOF(generalMesh,casMesh,snapMesh,layerMesh,surfNfeat,refinebox,snappyAdvDict)

            overwrite=generalMesh[0][3]

            SNAPPY.refineMeshDict()
            
            sf.write('topoSet -case '+self.caseDir+' -dict '+self.caseDir+'/system/topoSetDict.1'+'\n')
            sf.write('refineMesh -overwrite -case '+self.caseDir+' -dict '+self.caseDir+'/system/refineMeshDict'+'\n')
            sf.write('topoSet -case '+self.caseDir+' -dict '+self.caseDir+'/system/topoSetDict.2'+'\n')
            sf.write('refineMesh -overwrite -case '+self.caseDir+' -dict '+self.caseDir+'/system/refineMeshDict'+'\n')
            sf.write('topoSet -case '+self.caseDir+' -dict '+self.caseDir+'/system/topoSetDict.3'+'\n')
            sf.write('refineMesh -overwrite -case '+self.caseDir+' -dict '+self.caseDir+'/system/refineMeshDict'+'\n')
            sf.write('topoSet -case '+self.caseDir+' -dict '+self.caseDir+'/system/topoSetDict.4'+'\n')
            sf.write('refineMesh -overwrite -case '+self.caseDir+' -dict '+self.caseDir+'/system/refineMeshDict'+'\n')

            nCore=generalMesh[2]
            if nCore=='1':
                if overwrite=='false':
                    sf.write('snappyHexMesh -case '+self.caseDir+'\n')
                else:
                    sf.write('snappyHexMesh -overwrite -case '+self.caseDir+'\n')
            else:
                WFILE=writeFileClassResistance(self.caseDir,self.installpath)
                WFILE.decomposeParDict(nCore)
                sf.write('decomposePar -case '+self.caseDir+'\n')
                if self.hostfilename=='None':
                    if overwrite=='false':
                        sf.write('mpirun -np '+nCore+' snappyHexMesh -case '+self.caseDir+' -parallel\n')
                    else:
                        sf.write('mpirun -np '+nCore+' snappyHexMesh -overwrite -case '+self.caseDir+' -parallel\n')
                else:
                    if overwrite=='false':
                        sf.write('mpirun --hostfile '+self.hostfilename+' -np '+nCore+' snappyHexMesh -case '+self.caseDir+' -parallel\n')
                    else:
                        sf.write('mpirun --hostfile '+self.hostfilename+' -np '+nCore+' snappyHexMesh -overwrite -case '+self.caseDir+' -parallel\n')
                        
                if overwrite=='false':sf.write('reconstructParMesh -mergeTol 1e-6 -case '+self.caseDir+'\n')
                else:sf.write('reconstructParMesh -mergeTol 1e-6 -constant -case '+self.caseDir+'\n')
                sf.write('rm -r '+self.caseDir+'/processor*\n')
		
            sf.write('foamToVTK -case '+self.caseDir+'\n')               
            
            sf.close()
            os.system('chmod +x '+self.caseDir+'/runSnappy')
            cmd=self.caseDir+'/runSnappy\n'
            length=len(cmd)
            terminal.feed_child(cmd,length)            
                  
            """GEN=generalClass(self.caseDir);VTK=VTKStuff(self.caseDir)
            bcname=GEN.getBCName()
            onOff=[]
            for ii in bcname:onOff.append(1)
            onOff.append(0)
            viewposition=[1,1,1] 
            renderer=VTK.unsgeometry(bcname,onOff,viewposition)
            gvtk.GetRenderWindow().AddRenderer(renderer)
            gvtk.Initialize();gvtk.Start()              
            f=open(self.caseDir+'/system/settings/displayedPatch','w')
            for ii in onOff:f.write(str(ii)+'\n')
            f.close()
		    """
		     
            stime=time.asctime(time.localtime(time.time()))
            message='\n [  snappyHexMesh is started  at '+stime +'  ]\n\n\n'         
            enditer=textbuffer.get_end_iter()
            textbuffer.insert(enditer,message)
            
        def setSavedValue():
            fileName=self.caseDir+'/system/settings/snappyConditions';aa=glob.glob(fileName)            
            if len(aa)!=0:
                dic=GEN.makeDictFromFile(fileName)
                self.waterlineEntry.set_text(dic['waterline'])
                self.minxEntry.set_text(dic['minx']);self.minyEntry.set_text(dic['miny']);self.minzEntry.set_text(dic['minz'])
                self.maxxEntry.set_text(dic['maxx']);self.maxyEntry.set_text(dic['maxy']);self.maxzEntry.set_text(dic['maxz'])                          
                self.nodexEntry.set_text(dic['nodex']);self.nodeyEntry.set_text(dic['nodey']);self.nodezEntry.set_text(dic['nodez'])
                self.betweenEntry.set_text(dic['nCellsBetweenLevels'])
                self.tolEntry.set_text(dic['tolerance'])
                self.nsolveEntry.set_text(dic['nSolverIter'])
                self.nrelaxEntry.set_text(dic['nRelaxIter'])
                self.nlayerEntry.set_text(dic['nSurfaceLayers'])
                self.expandlayerEntry.set_text(dic['expansionRatio'])
                self.firstlayerEntry.set_text(dic['firstLayerThickness'])
                self.coreEntry.set_text(dic['nCores'])

        def selectstl(widget):

            stlfile=SNAPPY.selSTL()[0]
            f=open(self.caseDir+'/system/settings/stlfiles','w');f.write(stlfile);f.close()
            
            if glob.glob(self.caseDir+'/constant/triSurface')==[]:os.system('mkdir '+self.caseDir+'/constant/triSurface')
            if glob.glob(self.caseDir+'/constant/triSurface/*')!=[]:os.system('rm '+self.caseDir+'/constant/triSurface/*')         
            os.system('cp '+stlfile+' '+self.caseDir+'/constant/triSurface/')
            
            textbuffer.insert_at_cursor('\n -- '+stlfile+' is selected.\n\n')
            notebook.set_current_page(0) 
            
            speed=float(self.speed)
            waterline=float(self.waterlineEntry.get_text())
            os.system('surfaceCheck -case '+self.caseDir+' '+stlfile+' >'+self.caseDir+'/log.surfaceCheck')
            f=open(self.caseDir+'/log.surfaceCheck','r');ff=f.readlines();f.close()

            for ii in ff:
                if ii!='\n':
                    aa=ii.split()
                    if aa[0]=='Bounding':
        	        Xmin=float(aa[3][1:]);Ymin=float(aa[4]);Zmin=float(aa[5][:-1])
        	        Xmax=float(aa[6][1:]);Ymax=float(aa[7]);Zmax=float(aa[8][:-1])

            draft=waterline-Zmin
            freeboard=Zmax-waterline
            lbp=float(self.lbp)
            self.DT=draft
            self.FB=freeboard
            loa=lbp

            f=open(self.caseDir+'/system/settings/stlInfo','w')
            f.write(str(lbp)+' '+str(self.DT)+' '+str(self.FB));f.close()

            minimumX=str(-2*loa);maximumX=str(2*loa)
            minimumY='0';maximumY=str(1.5*loa)
            mmm=-1.5*loa+waterline
            minimumZ=("%.2f" % mmm)
            mama=4.5*freeboard+waterline
            maximumZ=("%.2f" % mama)	    

            self.minxEntry.set_text(minimumX);self.minyEntry.set_text(minimumY);self.minzEntry.set_text(minimumZ)
            self.maxxEntry.set_text(maximumX);self.maxyEntry.set_text(maximumY);self.maxzEntry.set_text(maximumZ)
            
            saveSnappy()

            p11=[str(-2.*loa),'0',str(-1.5*loa)]
            p12=[str(2.*loa),'0',str(-1.5*loa)]
            p13=[str(2.*loa),str(1.5*loa),str(-1.5*loa)]
            p14=[str(-2.*loa),str(1.5*loa),str(-1.5*loa)]

            p21=[str(-2.*loa),'0',str(-1.3*draft)]
            p22=[str(2.*loa),'0',str(-1.3*draft)]
            p23=[str(2.*loa),str(1.5*loa),str(-1.3*draft)]
            p24=[str(-2.*loa),str(1.5*loa),str(-1.3*draft)]

            p31=[str(-2.*loa),'0',str(-0.01*loa)]
            p32=[str(2.*loa),'0',str(-0.01*loa)]
            p33=[str(2.*loa),str(1.5*loa),str(-0.01*loa)]
            p34=[str(-2.*loa),str(1.5*loa),str(-0.01*loa)]

            p41=[str(-2.*loa),'0',str(0.01*loa)]
            p42=[str(2.*loa),'0',str(0.01*loa)]
            p43=[str(2.*loa),str(1.5*loa),str(0.01*loa)]
            p44=[str(-2.*loa),str(1.5*loa),str(0.01*loa)]

            p51=[str(-2.*loa),'0',str(2.0*freeboard)]
            p52=[str(2.*loa),'0',str(2.0*freeboard)]
            p53=[str(2.*loa),str(1.5*loa),str(2.0*freeboard)]
            p54=[str(-2.*loa),str(1.5*loa),str(2.0*freeboard)]

            p61=[str(-2.*loa),'0',str(4.5*freeboard)]
            p62=[str(2.*loa),'0',str(4.5*freeboard)]
            p63=[str(2.*loa),str(1.5*loa),str(4.5*freeboard)]
            p64=[str(-2.*loa),str(1.5*loa),str(4.5*freeboard)]

            p1=[p11,p12,p13,p14];p2=[p21,p22,p23,p24];p3=[p31,p32,p33,p34];p4=[p41,p42,p43,p44]
            p5=[p51,p52,p53,p54];p6=[p61,p62,p63,p64]

            topo1=[str(-1.25*loa),'0.0',str(-6.0*draft),str(0.6*loa),str(0.7*loa),str(6.0*freeboard)]
            topo2=[str(-1.2*loa),'0.0',str(-5.0*draft),str(0.5*loa),str(0.6*loa),str(5.0*freeboard)]
            topo3=[str(-1.15*loa),'0.0',str(-4.0*draft),str(0.4*loa),str(0.55*loa),str(4.5*freeboard)]
            topo4=[str(-1.1*loa),'0.0',str(-2.5*draft),str(0.3*loa),str(0.5*loa),str(2.5*freeboard)]

            SNAPPY.makeTopoSetDict(topo1,'topoSetDict.1')
            SNAPPY.makeTopoSetDict(topo2,'topoSetDict.2')
            SNAPPY.makeTopoSetDict(topo3,'topoSetDict.3')
            SNAPPY.makeTopoSetDict(topo4,'topoSetDict.4')
            #--------------------------
            yplus=50
            nu=0.000001;rho=1000.
            Re=loa*speed/nu
            Cf=0.075/(math.log10(Re)-2)**2
            tauw=Cf*0.5*rho*speed*speed
            ustar=math.sqrt(tauw/rho)
            yy=yplus*nu/ustar
            firstcell=2*yy;firstheight=("%.6f" % firstcell)
            minthickness=0.0001
            self.firstlayerEntry.set_text(str(firstheight))

            notebook.set_current_page(1)
   
            viewposition=[1,1,1] 
            renderer=VTK.showSTL(stlfile,viewposition,displayOptCombo.child.get_text())
            gvtk.GetRenderWindow().AddRenderer(renderer)
            gvtk.Initialize();gvtk.Start()              
            f=open(self.caseDir+'/system/settings/displayedPatch','w');f.write('STL');f.close()	    
            
        def blockMesh(widget):

            waterline=self.waterlineEntry.get_text();yplus='50';featureAngle='160'            
            domainMin=[self.minxEntry.get_text(),self.minyEntry.get_text(),self.minzEntry.get_text()]
            domainMax=[self.maxxEntry.get_text(),self.maxyEntry.get_text(),self.maxzEntry.get_text()]
            domainNode=[self.nodexEntry.get_text(),self.nodeyEntry.get_text(),self.nodezEntry.get_text()]
            
            blockDict={}
            blockDict['speed']=self.speed;blockDict['waterline']=waterline;blockDict['yplus']=yplus;blockDict['featureAngle']=featureAngle
            blockDict['minx']=domainMin[0];blockDict['miny']=domainMin[1];blockDict['minz']=domainMin[2]
            blockDict['maxx']=domainMax[0];blockDict['maxy']=domainMax[1];blockDict['maxz']=domainMax[2]
            blockDict['nodex']=domainNode[0];blockDict['nodey']=domainNode[1];blockDict['nodez']=domainNode[2]
            blockDict['lbp']=self.lbp
            
            f=open(self.caseDir+'/system/settings/blockmesh.txt','w')
            keys=blockDict.keys()
            for ii in keys:f.write(ii+';'+blockDict[ii]+'\n')
            f.close()

            fff=float(domainMax[2])/4.5;fb=float(("%.2f" % fff))

            aa=glob.glob(self.caseDir+'/system/settings/stlInfo')
            if len(aa)!=0:
                f=open(aa[0],'r');ff=f.readline();f.close()
                a1=ff.split(' ');self.DT=float(a1[1])
            dt=float(("%.2f" % self.DT))

            p11=[domainMin[0],domainMin[1],str(-1.5*float(self.lbp)+float(waterline))]
            p12=[domainMax[0],domainMin[1],str(-1.5*float(self.lbp)+float(waterline))]
            p13=[domainMax[0],domainMax[1],str(-1.5*float(self.lbp)+float(waterline))]
            p14=[domainMin[0],domainMax[1],str(-1.5*float(self.lbp)+float(waterline))]

            p21=[domainMin[0],domainMin[1],str(-1.3*dt+float(waterline))]
            p22=[domainMax[0],domainMin[1],str(-1.3*dt+float(waterline))]
            p23=[domainMax[0],domainMax[1],str(-1.3*dt+float(waterline))]
            p24=[domainMin[0],domainMax[1],str(-1.3*dt+float(waterline))]

            p31=[domainMin[0],domainMin[1],str(-0.01*float(self.lbp)+float(waterline))]
            p32=[domainMax[0],domainMin[1],str(-0.01*float(self.lbp)+float(waterline))]
            p33=[domainMax[0],domainMax[1],str(-0.01*float(self.lbp)+float(waterline))]
            p34=[domainMin[0],domainMax[1],str(-0.01*float(self.lbp)+float(waterline))]

            p41=[domainMin[0],domainMin[1],str(0.01*float(self.lbp)+float(waterline))]
            p42=[domainMax[0],domainMin[1],str(0.01*float(self.lbp)+float(waterline))]
            p43=[domainMax[0],domainMax[1],str(0.01*float(self.lbp)+float(waterline))]
            p44=[domainMin[0],domainMax[1],str(0.01*float(self.lbp)+float(waterline))]

            p51=[domainMin[0],domainMin[1],str(2.0*fb+float(waterline))]
            p52=[domainMax[0],domainMin[1],str(2.0*fb+float(waterline))]
            p53=[domainMax[0],domainMax[1],str(2.0*fb+float(waterline))]
            p54=[domainMin[0],domainMax[1],str(2.0*fb+float(waterline))]

            p61=[domainMin[0],domainMin[1],str(4.5*fb+float(waterline))]
            p62=[domainMax[0],domainMin[1],str(4.5*fb+float(waterline))]
            p63=[domainMax[0],domainMax[1],str(4.5*fb+float(waterline))]
            p64=[domainMin[0],domainMax[1],str(4.5*fb+float(waterline))]

            p1=[p11,p12,p13,p14];p2=[p21,p22,p23,p24];p3=[p31,p32,p33,p34];p4=[p41,p42,p43,p44]
            p5=[p51,p52,p53,p54];p6=[p61,p62,p63,p64]

            aa=glob.glob(self.caseDir+'/constant/polyMesh/*')
            if aa!=[]:os.system('rm -r '+self.caseDir+'/constant/polyMesh/*')
            
            aa=glob.glob(self.caseDir+'/VTK')
            if aa!=[]:os.system('rm -r '+self.caseDir+'/VTK')
            
            SNAPPY.makeBlockMeshVOFDict(p1,p2,p3,p4,p5,p6,domainNode)
            os.system('blockMesh -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.blockMesh')	

            cmd='cat '+self.caseDir+'/logfiles/log.blockMesh\n'
            length=len(cmd)
            terminal.feed_child(cmd,length)
            
            notebook.set_current_page(1)
                                     
            renderer=VTK.blockMesh()
            gvtk.GetRenderWindow().AddRenderer(renderer)
            gvtk.Initialize();gvtk.Start()              
            f=open(self.caseDir+'/system/settings/displayedPatch','w');f.write('blockMesh');f.close()

        def advSetup(widget):
            SNAPPY.advancedSetup(self.mainwindow)        
            
        def sizeLevel(widget):
            SNAPPY.sizeLevel(self.mainwindow)
        
        self.snappybox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  snappyHexMesh  </b>')
        ebox=gtk.EventBox();self.snappybox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label)        

        label=gtk.Label();label.set_markup('<i>  Geometry / BlockMesh  </i>');label.set_alignment(0,0.5)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)                      
        
        label=gtk.Label('waterline');label.set_alignment(0,0.5)
        self.waterlineEntry=gtk.Entry();self.waterlineEntry.set_size_request(60,28);self.waterlineEntry.set_text('0')        
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5);hbox.pack_start(self.waterlineEntry,False,False,5)
                
        label=gtk.Label('Base mesh');label.set_alignment(0,0.5)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_start(label,False,False,5)  
        
        label=gtk.Label('min.');label.set_alignment(0,0.5)
        self.minxEntry=gtk.Entry();self.minxEntry.set_size_request(60,28);self.minxEntry.set_text('-100')
        self.minyEntry=gtk.Entry();self.minyEntry.set_size_request(60,28);self.minyEntry.set_text('-100')
        self.minzEntry=gtk.Entry();self.minzEntry.set_size_request(60,28);self.minzEntry.set_text('-100')

        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_end(self.minzEntry,False,False,5);hbox.pack_end(self.minyEntry,False,False,5);hbox.pack_end(self.minxEntry,False,False,5)
        hbox.pack_end(label,False,False,5)
        
        label=gtk.Label('max.');label.set_alignment(0,0.5)
        self.maxxEntry=gtk.Entry();self.maxxEntry.set_size_request(60,28);self.maxxEntry.set_text('100')
        self.maxyEntry=gtk.Entry();self.maxyEntry.set_size_request(60,28);self.maxyEntry.set_text('100')
        self.maxzEntry=gtk.Entry();self.maxzEntry.set_size_request(60,28);self.maxzEntry.set_text('0')
              
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.maxzEntry,False,False,5);hbox.pack_end(self.maxyEntry,False,False,5);hbox.pack_end(self.maxxEntry,False,False,5)
        hbox.pack_end(label,False,False,5)
        
        label=gtk.Label('nodes');label.set_alignment(0,0.5)
        self.nodexEntry=gtk.Entry();self.nodexEntry.set_size_request(60,28);self.nodexEntry.set_text('20')
        self.nodeyEntry=gtk.Entry();self.nodeyEntry.set_size_request(60,28);self.nodeyEntry.set_text('9')
        self.nodezEntry=gtk.Entry();self.nodezEntry.set_size_request(60,28);self.nodezEntry.set_text('57')
        self.nodezEntry.set_sensitive(0)
                      
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_end(self.nodezEntry,False,False,5);hbox.pack_end(self.nodeyEntry,False,False,5);hbox.pack_end(self.nodexEntry,False,False,5)
        hbox.pack_end(label,False,False,5)        
        
        self.stlB=gtk.Button('STL file');self.stlB.set_size_request(120,28)
        self.stlB.connect('clicked',selectstl)
        self.blockMeshB=gtk.Button('BlockMesh');self.blockMeshB.set_size_request(120,28)
        self.blockMeshB.connect('clicked',blockMesh)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_end(self.blockMeshB,False,False,5);hbox.pack_end(self.stlB,False,False,5)          
        
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.snappybox.pack_start(separator,False,True,5)
        
        label=gtk.Label();label.set_markup('<i>  Castellate / Snap  </i>');label.set_alignment(0,0.5)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_start(label,False,False,5) 
                  
        self.sizeLevelB=gtk.Button('Set Size Level');self.sizeLevelB.set_size_request(150,28);self.sizeLevelB.connect('clicked',sizeLevel)           
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,0)
        hbox.pack_end(self.sizeLevelB,False,False,5)        

        label=gtk.Label('Cells Betwen levels');label.set_alignment(0,0.5)
        self.betweenEntry=gtk.Entry();self.betweenEntry.set_size_request(80,28);self.betweenEntry.set_text('5')           
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.betweenEntry,False,False,5);hbox.pack_end(label,False,False,5) 
                
        label=gtk.Label('tolerance');label.set_alignment(0,0.5)
        self.tolEntry=gtk.Entry();self.tolEntry.set_size_request(80,28);self.tolEntry.set_text('1')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.tolEntry,False,False,5);hbox.pack_end(label,False,False,5) 

        label=gtk.Label('nSolverIter');label.set_alignment(0,0.5)
        self.nsolveEntry=gtk.Entry();self.nsolveEntry.set_size_request(80,28);self.nsolveEntry.set_text('30')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.nsolveEntry,False,False,5);hbox.pack_end(label,False,False,5) 
        
        label=gtk.Label('nRelaxIter');label.set_alignment(0,0.5)
        self.nrelaxEntry=gtk.Entry();self.nrelaxEntry.set_size_request(80,28);self.nrelaxEntry.set_text('20')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.nrelaxEntry,False,False,5);hbox.pack_end(label,False,False,5) 
        
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.snappybox.pack_start(separator,False,True,2)
        
        label=gtk.Label();label.set_markup('<i>  Boundary Layers  </i>');label.set_alignment(0,0.5)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)                 
        
        label=gtk.Label('No. of layers');label.set_alignment(0,0.5)
        self.nlayerEntry=gtk.Entry();self.nlayerEntry.set_size_request(80,28);self.nlayerEntry.set_text('3')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.nlayerEntry,False,False,5);hbox.pack_end(label,False,False,5)        

        label=gtk.Label('Expansion ratio');label.set_alignment(0,0.5)
        self.expandlayerEntry=gtk.Entry();self.expandlayerEntry.set_size_request(80,28);self.expandlayerEntry.set_text('1.3')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.expandlayerEntry,False,False,5);hbox.pack_end(label,False,False,5)         
       
        label=gtk.Label('1st layer thickness');label.set_alignment(0,0.5)
        self.firstlayerEntry=gtk.Entry();self.firstlayerEntry.set_size_request(80,28);self.firstlayerEntry.set_text('0.0005')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.firstlayerEntry,False,False,5);hbox.pack_end(label,False,False,5)          
        
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.snappybox.pack_start(separator,False,True,2)
        
        label=gtk.Label('No. of cores');label.set_alignment(0,0.5)
        self.coreEntry=gtk.Entry();self.coreEntry.set_size_request(80,28);self.coreEntry.set_text('1')
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,2)
        hbox.pack_end(self.coreEntry,False,False,0);hbox.pack_end(label,False,False,5)
        
        self.advancedB=gtk.Button('Advanced Set');self.advancedB.connect('clicked',advSetup);self.advancedB.set_size_request(150,28)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,5);hbox.pack_end(self.advancedB,False,False,5)
        self.AppB=gtk.Button('Create Mesh');self.AppB.connect('clicked',createMesh)
        hbox=gtk.HBox(False,5);self.snappybox.pack_start(hbox,False,False,5);hbox.pack_start(self.AppB,True,True,5)
        
        setSavedValue()

        return self.snappybox



