#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for turbulence free-panel
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import os,math,sys;import pygtk;pygtk.require('2.0');import gtk;import glob

from generalSub import *
from boundaryConditionSub import *

class freepanelResistanceClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,mainwindow):
        self.caseDir=caseDir
        self.mainwindow=mainwindow

    def simConditions(self,notebook,textbuffer):
    
        GEN=generalClass(self.caseDir)

        def saveTur(widget):
            GEN=generalClass(self.caseDir)
            genDict={}
            genDict['speed']=self.speedEntry.get_text();genDict['lbp']=self.lbpEntry.get_text()
            genDict['aref']=self.arefEntry.get_text()            
            
            turmodel=self.turCombo.child.get_text()
            if turmodel=='Standard k-ε':genDict['RANSModel']='kEpsilon'
            elif turmodel=='realizable k-ε':genDict['RANSModel']='realizableKE'
            elif turmodel=='RNG k-ε':genDict['RANSModel']='RNGkE'
            elif turmodel=='k-ω':genDict['RANSModel']='kOmega'
            elif turmodel=='k-ω SST':genDict['RANSModel']='kOmegaSST'

            genDict['inletk']=self.kinEntry.get_text();genDict['inlete']=self.einEntry.get_text()
            
            genDict['rhol']=self.rholEntry.get_text();genDict['mul']=self.mulEntry.get_text()
            genDict['rhog']=self.rhogEntry.get_text();genDict['mug']=self.mugEntry.get_text()
            
            genDict['gx']=self.xgravEntry.get_text();genDict['gy']=self.ygravEntry.get_text();genDict['gz']=self.zgravEntry.get_text()

            if GEN.isNumber(genDict['speed'])==False:GEN.makeDialog('Error!!! Speed is not number')
            if GEN.isNumber(genDict['lbp'])==False:GEN.makeDialog('Error!!! lbp is not number') 
            if GEN.isNumber(genDict['aref'])==False:GEN.makeDialog('Error!!! Ref. area is not number')            
            if GEN.isNumber(genDict['inletk'])==False:GEN.makeDialog('Error!!! inlet k is not number') 
            if GEN.isNumber(genDict['inlete'])==False:GEN.makeDialog('Error!!! inlet ε/ω is not number')            
            if GEN.isNumber(genDict['rhol'])==False:GEN.makeDialog('Error!!! water density is not number') 
            if GEN.isNumber(genDict['mul'])==False:GEN.makeDialog('Error!!! water viscosity is not number') 
            if GEN.isNumber(genDict['rhog'])==False:GEN.makeDialog('Error!!! air density is not number') 
            if GEN.isNumber(genDict['mug'])==False:GEN.makeDialog('Error!!! air viscosity is not number')            
            if GEN.isNumber(genDict['gx'])==False:GEN.makeDialog('Error!!! gravity-x is not number') 
            if GEN.isNumber(genDict['gy'])==False:GEN.makeDialog('Error!!! gravity-y is not number') 
            if GEN.isNumber(genDict['gz'])==False:GEN.makeDialog('Error!!! gravity-z is not number')  
            
            keys=genDict.keys()
            f=open(self.caseDir+'/system/settings/generalConditions','w')
            for ii in keys:f.write(ii+';'+genDict[ii]+'\n')         
            f.close()
            #------------            
            message='\n [  Simulation Conditions are saved  ]\n' 
            message=message+'\n    - Ship Speed is '+genDict['speed']+'\n'
            message=message+'\n    - LBP is '+genDict['lbp']+'\n'
            message=message+'\n    - Reference Area is '+genDict['aref']+'\n'
            message=message+'\n    - Turbulence model is '+genDict['RANSModel']+'\n'
            message=message+'\n    - Inlet k is '+genDict['inletk']+'\n'
            message=message+'\n    - Inlet epsilon/omega is '+genDict['inlete']+'\n'
            message=message+'\n    - Gravity direction is ('+genDict['gx']+' '+genDict['gy']+' '+genDict['gz']+')\n'
            message=message+'\n    - Water Density is '+genDict['rhol']+'\n'
            message=message+'\n    - Water Viscosity is '+genDict['mul']+'\n'
            message=message+'\n    - Air Density is '+genDict['rhog']+'\n'
            message=message+'\n    - Air Viscosity is '+genDict['mug']+'\n'

            enditer=textbuffer.get_end_iter()
            textbuffer.insert(enditer,message)
            notebook.set_current_page(0)                 
            
        def setSavedValue():
            fileName=self.caseDir+'/system/settings/generalConditions';aa=glob.glob(fileName)
            if len(aa)!=0:
                GEN=generalClass(self.caseDir);genDict=GEN.makeDictFromFile(fileName)
                self.speedEntry.set_text(genDict['speed'])
                self.lbpEntry.set_text(genDict['lbp'])
                self.arefEntry.set_text(genDict['aref'])                 
                if genDict['RANSModel']=='kEpsilon':self.turCombo.set_active(0)
                elif genDict['RANSModel']=='realizableKE':self.turCombo.set_active(1)
                elif genDict['RANSModel']=='RNGkE':self.turCombo.set_active(2)
                elif genDict['RANSModel']=='kOmega':self.turCombo.set_active(4)
                elif genDict['RANSModel']=='kOmegaSST':self.turCombo.set_active(3)
                self.kinEntry.set_text(genDict['inletk']);self.einEntry.set_text(genDict['inlete'])
                self.rholEntry.set_text(genDict['rhol']);self.mulEntry.set_text(genDict['mul'])
                self.rhogEntry.set_text(genDict['rhog']);self.mugEntry.set_text(genDict['mug'])
                self.xgravEntry.set_text(genDict['gx']);self.ygravEntry.set_text(genDict['gy']);self.zgravEntry.set_text(genDict['gz'])

        self.conditionbox=gtk.VBox(False,5)
        label=gtk.Label();label.set_markup('<b>  Simulation conditions  </b>')
        ebox=gtk.EventBox();self.conditionbox.pack_start(ebox,True,True,0)
        label.set_alignment(0,0.5);label.set_size_request(50,45)
        ebox.modify_bg(gtk.STATE_NORMAL,gtk.gdk.color_parse('white'));ebox.add(label)
        
        label=gtk.Label('Speed');label.set_alignment(0,0.5);label.set_size_request(100,28)
        self.speedEntry=gtk.Entry();self.speedEntry.set_size_request(80,28);self.speedEntry.set_text('1')        
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5);hbox.pack_start(self.speedEntry,False,False,5)
        
        label=gtk.Label('LBP');label.set_alignment(0,0.5);label.set_size_request(100,28)
        self.lbpEntry=gtk.Entry();self.lbpEntry.set_size_request(80,28);self.lbpEntry.set_text('7.3')        
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,0)               
        hbox.pack_start(label,False,False,5);hbox.pack_start(self.lbpEntry,False,False,5)
        
        label=gtk.Label('Ref. Area');label.set_alignment(0,0.5);label.set_size_request(100,28)
        self.arefEntry=gtk.Entry();self.arefEntry.set_size_request(80,28);self.arefEntry.set_text('1')        
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,0)               
        hbox.pack_start(label,False,False,5);hbox.pack_start(self.arefEntry,False,False,5)       
        
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.conditionbox.pack_start(separator,True,True,10)
        #-----------------------        
        label=gtk.Label();label.set_markup('<i>  Turbulence Conditions  </i>');self.conditionbox.pack_start(label,False,False,5);label.set_alignment(0,0)
        
        label=gtk.Label('tur. model');label.set_size_request(100,28);label.set_alignment(0,0.5)
            
        self.turCombo=gtk.combo_box_entry_new_text()
        self.turModels=['Standard k-ε','realizable k-ε','RNG k-ε','k-ω SST','k-ω']  
        for ii in self.turModels:self.turCombo.append_text(ii)
        self.turCombo.set_active(0)
        self.turCombo.set_size_request(160,28)
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5);hbox.pack_start(self.turCombo,False,False,2)

        kinlabel=gtk.Label('        inlet k');kinlabel.set_alignment(0,0.5);kinlabel.set_size_request(100,28)
        self.kinEntry=gtk.Entry();self.kinEntry.set_size_request(80,28);self.kinEntry.set_text('0.0001')        
        kinhbox=gtk.HBox(False,5);self.conditionbox.pack_start(kinhbox,False,False,0)
        kinhbox.pack_start(kinlabel,False,False,5);kinhbox.pack_start(self.kinEntry,False,False,5)

        einlabel=gtk.Label('        inlet ε/ω');einlabel.set_alignment(0,0.5);einlabel.set_size_request(100,28)
        self.einEntry=gtk.Entry();self.einEntry.set_size_request(80,28);self.einEntry.set_text('0.001')        
        einhbox=gtk.HBox(False,5);self.conditionbox.pack_start(einhbox,False,False,0)
        einhbox.pack_start(einlabel,False,False,5);einhbox.pack_start(self.einEntry,False,False,5)

        separator=gtk.HSeparator();separator.set_size_request(285,5);self.conditionbox.pack_start(separator,True,True,10)        
        #-----------------------        
        label=gtk.Label();label.set_markup('<i>  Material Properties  </i>');self.conditionbox.pack_start(label,False,False,5);label.set_alignment(0,0)   
        
        label=gtk.Label('       Water Properties');label.set_alignment(0,0.5)      
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)
        
        label=gtk.Label('density [kg/m3]')
        self.rholEntry=gtk.Entry();self.rholEntry.set_size_request(80,28);self.rholEntry.set_text('1000')
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_end(self.rholEntry,False,False,5);hbox.pack_end(label,False,False,5)

        label=gtk.Label('viscosity [kg/ms]')
        self.mulEntry=gtk.Entry();self.mulEntry.set_size_request(80,28);self.mulEntry.set_text('0.001')
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_end(self.mulEntry,False,False,5);hbox.pack_end(label,False,False,5)
                
        label=gtk.Label('       Air Properties');label.set_alignment(0,0.5)      
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_start(label,False,False,5)
        
        label=gtk.Label('density [kg/m3]')
        self.rhogEntry=gtk.Entry();self.rhogEntry.set_size_request(80,28);self.rhogEntry.set_text('1.225')
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_end(self.rhogEntry,False,False,5);hbox.pack_end(label,False,False,5)

        label=gtk.Label('viscosity [kg/ms]')
        self.mugEntry=gtk.Entry();self.mugEntry.set_size_request(80,28);self.mugEntry.set_text('1e-5')
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_end(self.mugEntry,False,False,5);hbox.pack_end(label,False,False,5)
        
        separator=gtk.HSeparator();separator.set_size_request(285,5);self.conditionbox.pack_start(separator,True,True,10)        
        #-----------------------        
        label=gtk.Label();label.set_markup('<i>  Gravity  </i>');self.conditionbox.pack_start(label,False,False,5);label.set_alignment(0,0)         

        self.xgravEntry=gtk.Entry();self.xgravEntry.set_size_request(60,28);self.xgravEntry.set_text('0')
        self.ygravEntry=gtk.Entry();self.ygravEntry.set_size_request(60,28);self.ygravEntry.set_text('0')
        self.zgravEntry=gtk.Entry();self.zgravEntry.set_size_request(60,28);self.zgravEntry.set_text('-9.81')

        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,5)
        hbox.pack_end(self.zgravEntry,False,False,2);hbox.pack_end(self.ygravEntry,False,False,2);hbox.pack_end(self.xgravEntry,False,False,2)

        label=gtk.Label('   g [m/s2]');hbox.pack_start(label,False,False,5)
                        
        # ---------------------
        hbox=gtk.HBox(False,5);self.conditionbox.pack_start(hbox,False,False,10)
        self.turbulenceAppB=gtk.Button('Apply');hbox.pack_end(self.turbulenceAppB,True,True,5)
        self.turbulenceAppB.connect('clicked',saveTur)
        
        setSavedValue()
        
        return self.conditionbox




