#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for run RESOS solver
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################
import os,math,glob,sys;import pygtk;pygtk.require('2.0');import gtk
import time

from generalSub import *
from changeDict import *
from misc import *
from writeFileResistanceSub import *

class resistanceSolvingClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir,installpath,terminal):
        self.caseDir=caseDir
        self.installpath=installpath
        self.terminal=terminal
    #----------------------------------------------------------------------------------    
    def runSolver(self,inputDictList,solver,mode,notebook,textbuffer):

        #inputDictList=[turDict,propertyDict,initialDict,runControlDict,cellZoneDictList,bcDictList,gravityDict]
        bcDictList=inputDictList[5]
        turDict=inputDictList[0]           

        turbulence=turDict['RANSModel']

        runControlDict=inputDictList[3]
       
        nCore=runControlDict['nCores'];startFrom=runControlDict['startFrom'];plotresidual=runControlDict['plotResidual']
        smpOrCluster=runControlDict['parallelType'];hostfile=runControlDict['hostfile'];startTime=runControlDict['startTime']

        GEN=generalClass(self.caseDir);WFILE=writeFileClassResistance(self.caseDir,self.installpath);MISC=miscClass(self.caseDir)
	
        ss=glob.glob(self.caseDir+'/Running')
        if len(ss)==0:
            if mode=='initialize':
                MISC.UFileBasic()
                MISC.p_rghFileBasic(bcDictList)
                MISC.alphaFileBasic()
                
                MISC.kFileBasic()
                if turbulence=='kOmega' or turbulence=='kOmegaSST':MISC.omegaFileBasic()
                else:MISC.epsilonFileBasic()		
                savedResult=GEN.getResult()
                for ii in savedResult[0]:os.system('rm -r '+self.caseDir+'/'+ii)
                if len(savedResult[1])!=0:os.system('rm -r '+self.caseDir+'/processor*')
                self.solverSet(inputDictList,turbulence,solver,mode,notebook,textbuffer)
            elif mode=='potentialFoam':
                self.solverSet(inputDictList,turbulence,solver,mode,notebook,textbuffer)            
            else:
                self.solverSet(inputDictList,turbulence,solver,mode,notebook,textbuffer)
                self.solverRun(solver,startFrom,startTime,nCore,plotresidual,smpOrCluster,hostfile,notebook,textbuffer)
        else:
            md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Solver is Running")
            md.run();md.destroy()
    #----------------------------------------------------------------------------------
    def solverSet(self,inputDictList,turbulence,solver,mode,notebook,textbuffer):

        GEN=generalClass(self.caseDir);WFILE=writeFileClassResistance(self.caseDir,self.installpath);CHDICT=changeDictionary(self.caseDir)	

        turDict=inputDictList[0];propertyDict=inputDictList[1]
        initialDict=inputDictList[2];runControlDict=inputDictList[3];cellZoneDictList=inputDictList[4];bcDictList=inputDictList[5]
        gravityDict=inputDictList[6]

        gg=gravityDict['gravity']

        nCore=runControlDict['nCores'];startFrom=runControlDict['startFrom'];plotresidual=runControlDict['plotResidual']
        smpOrCluster=runControlDict['parallelType'];hostfile=runControlDict['hostfile'];startTime=runControlDict['startTime']
      
        density='1000'
        
        if mode=='initialize':
            CHDICT.changeInternalField(initialDict,'VOF',notebook,textbuffer)                       
            CHDICT.UFile(bcDictList,startFrom,startTime,nCore,density,initialDict,notebook,textbuffer)                      
            CHDICT.p_rghFileVOF(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            CHDICT.alphaFileVOF(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            CHDICT.kFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            if turbulence=='kOmega' or turbulence=='kOmegaSST':CHDICT.omegaFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            else:CHDICT.epsilonFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
        else:
            savedResult=GEN.getResult()
            if savedResult[0]!=[] or savedResult[1]!=[]:
                CHDICT.changeInternalField(initialDict,'VOF',notebook,textbuffer)                       
                CHDICT.UFile(bcDictList,startFrom,startTime,nCore,density,initialDict,notebook,textbuffer)                      
                CHDICT.p_rghFileVOF(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                CHDICT.alphaFileVOF(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                CHDICT.kFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                if turbulence=='kOmega' or turbulence=='kOmegaSST':CHDICT.omegaFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
                else:CHDICT.epsilonFile(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            
        WFILE.gravity(gg)
        WFILE.RASProperties(turbulence)
        WFILE.transportPropertiesVOF(propertyDict)  
        WFILE.turbulenceProperties()        
        WFILE.controlDictFile(runControlDict,solver)        
        WFILE.decomposeParDict(nCore)

        po=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Porous');mr=glob.glob(self.caseDir+'/system/cellZoneConditions:*:MRF')
        fu=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Fixed.U');ms=glob.glob(self.caseDir+'/system/cellZoneConditions:*:Mom.Source')
        hs=glob.glob(self.caseDir+'/system/cellZoneConditions:*:None')
        WFILE.fvOptions(po,mr,fu,ms,hs)
        
        if mode=='initialize':
            aa=glob.glob(self.caseDir+'/0.org')
            if len(aa)==0:
                os.system('cp -r '+self.caseDir+'/0 '+self.caseDir+'/0.org')
            else:
                os.system('rm -r '+self.caseDir+'/0.org')
                os.system('cp -r '+self.caseDir+'/0 '+self.caseDir+'/0.org')
            
            os.system('setFields -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.setFields')            
            cmd='cat '+self.caseDir+'/logfiles/log.setFields\n'
            length=len(cmd)
            self.terminal.feed_child(cmd,length)

        if mode=='potentialFoam':
            MISC=miscClass(self.caseDir);MISC.pFileBasicPotential()
            CHDICT.pFilePotential(bcDictList,startFrom,startTime,nCore,initialDict,notebook,textbuffer)
            os.system('potentialFoam -noFunctionObjects -case '+self.caseDir+' > '+self.caseDir+'/logfiles/log.potentialFoam')
            GEN.createlog('log.potentialFoam',textbuffer)
            os.system('mv '+self.caseDir+'/0/p '+self.caseDir+'/0/p.potential')
    #----------------------------------------------------------------------------------
    def solverRun(self,solver,startFrom,startTime,nCore,plotresidual,smpOrCluster,hostfile,notebook,textbuffer):

        GEN=generalClass(self.caseDir);results=GEN.getResult();serialResult=results[0];parallelResult=results[1]

        notebook.set_current_page(2) 
        
        rf=open(self.caseDir+'/runSolver','w')
        
        aa=glob.glob(self.caseDir+'/postProcessing')
        if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/postProcessing \n')

        aa=glob.glob(self.caseDir+'/PyFoam*')
        if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/PyFoam* \n')
	    
        if nCore=='1':#serial
            if len(parallelResult)==0:GEN.clearParallel()
            else:
                rf.write('reconstructPar -case /'+self.caseDir+' -latestTime \n')
                aa=glob.glob(self.caseDir+'/processor*')
                if len(aa)!=0:rf.write('rm -r '+self.caseDir+'/processor* \n')
            if plotresidual=='true':
                rf.write('pyFoamPlotRunner.py --no-continuity '+solver+' -case '+self.caseDir+'\n')
            else:
                rf.write('pyFoamRunner.py '+solver+' -case '+self.caseDir+'\n')

        else:#parallel
            aa=glob.glob(self.caseDir+'/processor*');nCorePrev=str(len(aa))
            if nCore==nCorePrev:
                GEN.clearResult(serialResult,'0')
                if smpOrCluster=='SMP':
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                    else:
                        rf.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                else:
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                    else:
                        rf.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')

            else:#nCore!=nCorePrev
                if nCorePrev=='0':rf.write('decomposePar -force -case '+self.caseDir+' \n')
                else:
                    GEN.clearResult(serialResult,'0')
                    rf.write('reconstructPar -latestTime -case '+self.caseDir+' \n')
                    rf.write('decomposePar -force -case '+self.caseDir+' \n')
                if smpOrCluster=='SMP':
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                    else:
                        rf.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                else:
                    if plotresidual=='true':
                        rf.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
                    else:
                        rf.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel\n')
        
        rf.close()
        os.system('chmod +x '+self.caseDir+'/runSolver')
        cmd=self.caseDir+'/runSolver\n'
        length=len(cmd)
        self.terminal.feed_child(cmd,length)  

        stime=time.asctime(time.localtime(time.time()))
        message='\n [  Solving is started  at '+stime +'  ]\n\n\n'         
        enditer=textbuffer.get_end_iter()
        textbuffer.insert(enditer,message)
    #----------------------------------------------------------------------------------






         

  


