#-*-coding:utf8-*-
#############################################################################
# License                                                                       
#    This file is part of RESOS(RESistance Open Source CFD package).                                            
#
#    RESOS is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    RESOS is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with RESOS.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for writing various files of RESOS solver
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import math,os,glob,sys

from generalSub import *

class writeFileClassResistance:

    def __init__(self,caseDir,installpath):
        self.caseDir=caseDir
        self.installpath=installpath
	
    #-------------------------------------------------------------------------------------
    def foamComment(self):
        fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
        return fc
    #------------------------------------------------------------------------------
    def header(self,location,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "'+location+'";\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def BCheader(self,classtype,obj):
        he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       '+classtype+';\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
        he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
        return he
    #------------------------------------------------------------------------------
    def RASProperties(self,turModel):        
        fcom=self.foamComment();head=self.header('constant','RASProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/RASProperties','w')
        for aaa in hhh:f.write(aaa)	
        f.write('RASModel        '+turModel+';\n')
        f.write('turbulence      on;\n')
        f.write('printCoeffs     on;\n')       
        f.close()
    #------------------------------------------------------------------------------
    def turbulenceProperties(self):
        fcom=self.foamComment();head=self.header('constant','turbulenceProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/turbulenceProperties','w')
        for aaa in hhh:f.write(aaa)	
        f.write('simulationType        RASModel;\n')
        f.close()
    #------------------------------------------------------------------------------
    def transportProperties(self,viscosity,density):
        fcom=self.foamComment();head=self.header('constant','transportProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/transportProperties','w')
        for aaa in hhh:f.write(aaa)
        f.write('transportModel    Newtonian;\n')
        nu=float(viscosity)/float(density)
        f.write('nu    nu [ 0 2 -1 0 0 0 0 ] '+str(nu)+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def transportPropertiesVOF(self,dic):
        fcom=self.foamComment();head=self.header('constant','transportProperties');hhh=fcom+head
        f=open(self.caseDir+'/constant/transportProperties','w')
        for aaa in hhh:f.write(aaa)

        nug=float(dic['mug'])/float(dic['rhog'])
        nul=float(dic['mul'])/float(dic['rhol'])
        
        f.write('transportModel    Newtonian;\n')        
        f.write('    nu              nu [ 0 2 -1 0 0 0 0 ]  '+str(nul)+';\n')
        
        f.write('phases (water air);\n')
        f.write('water\n')
        f.write('{\n')
        f.write('    transportModel  Newtonian;\n')
        f.write('    nu              nu [ 0 2 -1 0 0 0 0 ]  '+str(nul)+';\n')
        f.write('    rho             rho [ 1 -3 0 0 0 0 0 ] '+dic['rhol']+';\n')
        f.write('}\n')
        f.write('air\n')
        f.write('{\n')
        f.write('    transportModel  Newtonian;\n')
        f.write('    nu              nu [ 0 2 -1 0 0 0 0 ]  '+str(nug)+';\n')
        f.write('    rho             rho [ 1 -3 0 0 0 0 0 ] '+dic['rhog']+';\n')
        f.write('}\n')
        f.write('sigma           sigma [ 1 0 -2 0 0 0 0 ] 0;\n')
        f.close()        
    #------------------------------------------------------------------------------
    def dynamicMeshDict(self,sm):
        fcom=self.foamComment();head=self.header('constant','dynamicMeshDict');hhh=fcom+head
        f=open(self.caseDir+'/constant/dynamicMeshDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('dynamicFvMesh   multiSolidBodyMotionFvMesh;\n')
        f.write('motionSolverLibs ( "libfvMotionSolvers.so" );\n')
        f.write('\n')
        f.write('multiSolidBodyMotionFvMeshCoeffs\n')
        f.write('{\n')
        for ii in sm:f.write('    #include "'+ii+'"\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def decomposeParDict(self,ncore):
        fcom=self.foamComment();head=self.header('system','decomposeParDict');hhh=fcom+head
        f=open(self.caseDir+'/system/decomposeParDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('numberOfSubdomains     '+ncore+';\n')
        f.write('method                 scotch;\n')
        f.close()
    #------------------------------------------------------------------------------
    def controlDictFile(self,runControlDict,solver):
        f1=glob.glob(self.caseDir+'/system/probeFunction_*')
        f2=glob.glob(self.caseDir+'/system/surfaceProbeFunction')
        f3=glob.glob(self.caseDir+'/system/flowRateFunction')
        f4=glob.glob(self.caseDir+'/system/forces')
        f5=glob.glob(self.caseDir+'/system/forceCoeffs')

        fcom=self.foamComment();head=self.header('system','controlDict');hhh=fcom+head
        f=open(self.caseDir+'/system/controlDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('application        '+solver+';\n')
        f.write('startFrom          '+runControlDict['startFrom']+';\n')
        f.write('startTime          '+runControlDict['startTime']+';\n')
        f.write('stopAt             endTime;\n')
        f.write('endTime            '+runControlDict['endTime']+';\n')
        f.write('deltaT             '+runControlDict['timeStepSize']+';\n')
        f.write('writeControl       '+runControlDict['writeControl']+';\n')
        f.write('writeInterval      '+runControlDict['writeInterval']+';\n')
        f.write('purgeWrite         '+runControlDict['purgeWrite']+';\n')
        f.write('writeFormat        '+runControlDict['writeFormat']+';\n')
        f.write('writePrecision     '+runControlDict['writePrecision']+';\n')  
        f.write('writeCompression   '+runControlDict['dataCompression']+';\n')
        f.write('timeFormat         general;\n')
        f.write('timePrecision      6;\n')
        f.write('runTimeModifiable  yes;\n')
        
        f.write('adjustTimeStep     '+runControlDict['adjustTimeStep']+';\n')
        f.write('maxCo              '+runControlDict['maxCo']+';\n')
        f.write('maxAlphaCo         '+runControlDict['maxAlphaCo']+';\n')
        #f.write('maxDeltaT          1;\n')
        
        f.write('functions\n')
        f.write('{\n')
        if len(f1)!=0:
            for ii in f1:f.write('    #include "'+ii+'"\n')
        if len(f2)!=0:f.write('    #include "surfaceProbeFunction"\n')
        if len(f3)!=0:f.write('    #include "flowRateFunction"\n')
        if len(f4)!=0:f.write('    #include "forces"\n')
        if len(f5)!=0:f.write('    #include "forceCoeffs"\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def fvOptions(self,po,mr,fu,ms,hs):
        fcom=self.foamComment();head=self.header('system','fvOptions');hhh=fcom+head
        f=open(self.caseDir+'/system/fvOptions','w')
        for aaa in hhh:f.write(aaa)
        if len(po)!=0:
            for ii in po:f.write('#include "'+ii+'"\n')
        if len(mr)!=0:
            for ii in mr:f.write('#include "'+ii+'"\n')
        if len(fu)!=0:
            for ii in fu:f.write('#include "'+ii+'"\n')
        if len(ms)!=0:
            for ii in ms:f.write('#include "'+ii+'"\n')
        if len(hs)!=0:
            for ii in hs:f.write('#include "'+ii+'"\n')
        f.close()
    #------------------------------------------------------------------------------
    def gravity(self,gg):
        fcom=self.foamComment();head=self.header('constant','g');hhh=fcom+head
        f=open(self.caseDir+'/constant/g','w')
        for aaa in hhh:f.write(aaa)
        f.write('dimensions      [0 1 -2 0 0 0 0];\n')
        f.write('value           '+gg+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def sixDofDynamicMeshDict(self,dic):
        fcom=self.foamComment();head=self.header('constant','dynamicMeshDict');hhh=fcom+head
        f=open(self.caseDir+'/constant/dynamicMeshDict','w')
        for aaa in hhh:f.write(aaa)
        f.write('\n')
        f.write('dynamicFvMesh       dynamicMotionSolverFvMesh;\n')
        f.write('motionSolverLibs    ("libsixDoFRigidBodyMotion.so");\n')
        f.write('solver              sixDoFRigidBodyMotion;\n')
        f.write('sixDoFRigidBodyMotionCoeffs\n')
        f.write('{\n')
        f.write('    patches         ('+dic['hull']+');\n')
        f.write('    innerDistance   '+dic['innerDistance']+';\n')
        f.write('    outerDistance   '+dic['outerDistance']+'1;\n')
        f.write('\n')
        f.write('    centreOfMass    ('+dic['centerOfMassX']+' '+dic['centerOfMassY']+' '+dic['centerOfMassZ']+');\n')
        f.write('    mass            '+dic['mass']+';\n')
        f.write('    momentOfInertia ('+dic['momentOfInerialX']+' '+dic['momentOfInerialY']+' '+dic['momentOfInerialZ']+');\n')
        f.write('    rhoInf          '+dic['rhoInf']+';\n')
        f.write('    report          on;\n')
        f.write('    accelerationRelaxation '+dic['accelerationRelaxation']+';\n')
        f.write('    value           uniform (0 0 0);\n')
        f.write('\n')
        f.write('    constraints\n')
        f.write('    {\n')
        f.write('        zAxis\n')
        f.write('        {\n')
        f.write('            sixDoFRigidBodyMotionConstraint line;\n')
        f.write('            direction     (0 0 1);\n')
        f.write('        }\n')
        f.write('        yPlane\n')
        f.write('        {\n')
        f.write('            sixDoFRigidBodyMotionConstraint axis;\n')
        f.write('            axis          (0 1 0);\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('\n')
        f.write('    restraints\n')
        f.write('    {\n')
        f.write('        translationDamper\n')
        f.write('        {\n')
        f.write('            sixDoFRigidBodyMotionRestraint linearDamper;\n')
        f.write('            coeff         '+dic['translationDamper']+';\n')
        f.write('        }\n')
        f.write('        rotationDamper\n')
        f.write('        {\n')
        f.write('            sixDoFRigidBodyMotionRestraint sphericalAngularDamper;\n')
        f.write('            coeff         '+dic['rotationDamper']+';\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def pointDisplacement(self):
    
        GEN=generalClass(self.caseDir)            
        bcname=GEN.getBCName();bctype=GEN.getBCType()
        hulls=[]
        for i in range(len(bcname)):
            if bctype[i]=='wall':hulls.append(bcname[i])
        
        fcom=self.foamComment();head=self.BCheader('pointVectorField','pointDisplacement');hhh=fcom+head
        f=open(self.caseDir+'/0/pointDisplacement','w')
        for aaa in hhh:f.write(aaa)
        f.write('dimensions      [0 1 0 0 0 0 0];\n')
        f.write('internalField   uniform (0 0 0);\n')
        f.write('boundaryField\n')
        f.write('{\n')
        for i in range(len(bcname)):
            f.write('    '+bcname[i]+'\n')
            f.write('    {\n')
            if bctype[i]=='wall':whattype='calculated'
            elif bctype[i]=='patch':whattype='fixedValue'
            elif bctype[i]=='symmetry':whattype='symmetry'
            elif bctype[i]=='symmetryPlane':whattype='symmetryPlane'
            else:
                GEN.makeDialog('Error! '+bcname[i]+' is not wall or patch or symmetry')
            f.write('        type    '+whattype+';\n')
            f.write('        value    uniform (0 0 0);\n')
            f.write('    }\n')
        f.write('}\n')
        f.close()


            
            
        
        
        
        
        
