/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2012 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "alphaWeightedMixedFvPatchScalarField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "uniformDimensionedFields.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::alphaWeightedMixedFvPatchScalarField::
alphaWeightedMixedFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    mixedFvPatchScalarField(p, iF),
    phaseName_("alpha"),
    pRefValue_(0.0)
{
    this->refValue() = 0.0;
    this->refGrad() = 0.0;
    this->valueFraction() = 0.0;
}


Foam::alphaWeightedMixedFvPatchScalarField::
alphaWeightedMixedFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    mixedFvPatchScalarField(p, iF),
    phaseName_(dict.lookupOrDefault<word>("phaseName", "alpha")),
    pRefValue_(readScalar(dict.lookup("pRefValue")))
{
    this->refValue() = pRefValue_;

    if (dict.found("value"))
    {
        fvPatchScalarField::operator=
        (
            scalarField("value", dict, p.size())
        );
    }
    else
    {
        fvPatchScalarField::operator=(this->refValue());
    }

    this->refGrad() = 0.0;
    this->valueFraction() = 0.0;
}


Foam::alphaWeightedMixedFvPatchScalarField::
alphaWeightedMixedFvPatchScalarField
(
    const alphaWeightedMixedFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    mixedFvPatchScalarField(ptf, p, iF, mapper),
    phaseName_(ptf.phaseName_),
    pRefValue_(ptf.pRefValue_)
{}


Foam::alphaWeightedMixedFvPatchScalarField::
alphaWeightedMixedFvPatchScalarField
(
    const alphaWeightedMixedFvPatchScalarField& ptf
)
:
    mixedFvPatchScalarField(ptf),
    phaseName_(ptf.phaseName_)
{}


Foam::alphaWeightedMixedFvPatchScalarField::
alphaWeightedMixedFvPatchScalarField
(
    const alphaWeightedMixedFvPatchScalarField& ptf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    mixedFvPatchScalarField(ptf, iF),
    phaseName_(ptf.phaseName_),
    pRefValue_(ptf.pRefValue_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::alphaWeightedMixedFvPatchScalarField::updateCoeffs()
{
    if (this->updated())
    {
        return;
    }

    const scalarField& alphap =
        patch().lookupPatchField<volScalarField, scalar>
        (
            phaseName_
        );

//    valueFraction() = mag(max(min(alphap, scalar(1.0)), scalar(0.0)) - 0.5) * 5. ;

    valueFraction() = 0.;
    forAll(this->patch(),faceI)
    {
        if(alphap[faceI] > 0.99)
        {
            valueFraction()[faceI] = 1.;
        }
        if(alphap[faceI] < 0.01)
        {
            valueFraction()[faceI] = 1.;
        }
    }


    refValue() = pRefValue_;

    mixedFvPatchScalarField::updateCoeffs();
}


void Foam::alphaWeightedMixedFvPatchScalarField::write(Ostream& os) const
{
    fvPatchScalarField::write(os);
    if (phaseName_ != "alpha")
    {
        os.writeKeyword("phaseName")
            << phaseName_ << token::END_STATEMENT << nl;
    }
    os.writeKeyword("pRefValue") << pRefValue_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * Member Operators  * * * * * * * * * * * * * //

void Foam::alphaWeightedMixedFvPatchScalarField::operator=
(
    const fvPatchScalarField& ptf
)
{
    fvPatchScalarField::operator=
    (
        valueFraction()*refValue()
        + (1 - valueFraction())*ptf
    );
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        alphaWeightedMixedFvPatchScalarField
    );
}

// ************************************************************************* //
